/******************************************************************************
 * Module Name : $RCSfile: Application.java,v $
 *
 * Description : Main application for the stand-up comic collection program.
 *
 * Updates:
 *   $Log: Application.java,v $
 *   Revision 1.20  2003/01/23 07:19:20  jcouper
 *   Add debug support to Application class
 *
 *   Revision 1.19  2002/07/07 01:16:54  jcouper
 *   job:155
 *   Forward and Back keys in edit issue dialog.
 *
 *   Revision 1.18  2001/10/07 04:42:27  jcouper
 *   141: Now prompts for confirm removing filtered title ref
 *
 *   Revision 1.17  2001/10/07 03:57:35  jcouper
 *   121: Added up/down buttons for titleRef
 *
 *   Revision 1.16  2001/07/11 07:22:31  jcouper
 *   Add confirmation dialog for delete, and filter for titles
 *
 *   Revision 1.15  2001/07/09 01:45:11  jcouper
 *   Update version to 1.0.4, remove broken help links
 *
 *   Revision 1.14  2001/04/04 07:34:50  jcouper
 *   Support credits, filters, new columns, etc, etc.  Stable release.
 *
 *   Revision 1.13  2000/03/18 00:56:08  Jonathan
 *   Changed version to 1.0.2
 *
 *   Revision 1.12  2000/03/18 00:46:04  Jonathan
 *   Big changes in the way the IndexDialog works, including a new CardDialog.
 *   New preferences for column display.  Permit renaming of title files.
 *
 *   Revision 1.11  2000/02/19 00:18:48  Jonathan
 *   Add saving of settings.
 *
 *   Revision 1.10  2000/02/18 22:35:23  Jonathan
 *   Add basic application settings support.
 *
 *   Revision 1.9  2000/01/10 02:22:20  Jonathan
 *   v1.0.0
 *
 *   Revision 1.8  1999/11/27 10:31:29  Jonathan
 *   Interface tweaks.  Reporting on the menu.  Fooling around with HTML
 *   about box.
 *
 *   Revision 1.7  1999/11/15 22:26:52  Jonathan
 *   Report stuff compiles at least.  Doesn't run, mind you.
 *
 *   Revision 1.6  1999/10/25 08:33:07  Jonathan
 *   v0_9_2
 *
 *   Revision 1.5  1999/10/24 22:37:54  Jonathan
 *   Proper CVS headers.  Fix A-018/19/20
 *
 *
 * Copyright Notice:
 * (c) 1999. This source code was developed by Jonathan Couper and friends.
 *            You may use it for any purpose.  No guarantees given or implied.
 *****************************************************************************/
package standup;

import java.awt.*;
import javax.swing.*;

public class Application {
    static ApplicationSettings applicationSettings = null;
    static final int SPLASH_SECONDS = 3;
    static MainFrame mainFrame = null;
    static boolean debug = false;

    public static void main (String[] args) {
	for (int i=0; i<args.length; i++) {
	    if (args[i].equals("-debug")) {
		debug = true;

	    } else {
		System.out.println ("Notice: Unknown command line arg - " + args[i]);
	    }
	}

        mainFrame = new MainFrame();

        mainFrame.setVisible(true);

        // Size and place the splash window.
        Window splashWindow = new Window (mainFrame);
        ImageIcon splashIcon = new ImageIcon (mainFrame.getClass().getResource ("images/splash.gif"));
        JLabel splashLabel = new JLabel (splashIcon);
        Dimension momSize = mainFrame.getSize();
        Point momPos = mainFrame.getLocationOnScreen();

        splashWindow.add (splashLabel);
        splashWindow.pack();

        Dimension splashSize = splashWindow.getSize();
        splashWindow.setLocation (momPos.x + (momSize.width / 2) - (splashSize.width / 2),
                                  momPos.y + (momSize.height / 2) - (splashSize.height / 2));

        splashWindow.setVisible(true);
        splashWindow.toFront();

        // Done now, hide the splash.
        mainFrame.fillIndex(true);
        splashWindow.setVisible(false);
    }

    public static String getAppName () {
        return "Standup Comic Manager";
    }

    public static String getVersion () {
        return "1.0.6";
    }

    public static MainFrame getMainFrame () {
        return mainFrame;
    }

    public static ApplicationSettings getSettings () {
        if (applicationSettings == null) {
            applicationSettings = new ApplicationSettings ();
            applicationSettings.load ();
        }
        return applicationSettings;
    }

    public static void saveSettings () {
        if (applicationSettings != null) {
            applicationSettings.save ();
        }
    }

    public static void debug (String className, String functionName, String info) {
	if (!debug) return;
	System.out.println (className + ":" + functionName + ": " + info);
    }

    public static void debug (String className, String functionName, Exception e) {
	if (!debug) return;
	System.out.println (className + ":" + functionName);
	e.printStackTrace();
    }
}
