/******************************************************************************
 * Module Name : $RCSfile: ApplicationSettings.java,v $
 *
 * Description : Holds all the settings.  Some are maintained automatically,
 *               and some are maintained explicitly by the settings dialog.
 *
 * Updates:
 *   $Log: ApplicationSettings.java,v $
 *   Revision 1.6  2002/07/13 02:22:20  jcouper
 *   job:157
 *   Still tweaking after changing grading and valuation.
 *
 *   Revision 1.5  2002/07/07 00:50:30  jcouper
 *   job:152
 *   Add support for Unix file line endings.
 *
 *   Revision 1.4  2001/04/04 07:34:50  jcouper
 *   Support credits, filters, new columns, etc, etc.  Stable release.
 *
 *   Revision 1.3  2000/03/18 00:46:04  Jonathan
 *   Big changes in the way the IndexDialog works, including a new CardDialog.
 *   New preferences for column display.  Permit renaming of title files.
 *
 *   Revision 1.2  2000/02/19 00:18:48  Jonathan
 *   Add saving of settings.
 *
 *   Revision 1.1  2000/02/18 22:35:23  Jonathan
 *   Add basic application settings support.
 *
 *
 * Copyright Notice:
 * (c) 1999. This source code was developed by Jonathan Couper and friends.
 *            You may use it for any purpose.  No guarantees given or implied.
 *****************************************************************************/
package standup;

import java.io.*;
import java.util.*;
import standup.parse.*;

public class ApplicationSettings {
    File propertiesFile = new File ("standup.txt");
    String propertiesHeader = "Standup v" + Application.getVersion();
    Properties properties = new Properties ();

    static final String SETTINGS_BASE = "org.spiderfan.standup.";

    static final String SHOW_DATE_COLUMN = "showDateColumn";
    static final String SHOW_GRADE_COLUMN = "showGradeColumn";
    static final String SHOW_NM_VALUE_COLUMN = "showNmValueColumn";
    static final String SHOW_VALUE_COLUMN = "showValueColumn";
    static final String SHOW_INFO_COLUMN = "showInfoColumn";
    static final String SHOW_COMMENT_COLUMN = "showCommentColumn";
    public boolean showDateColumn = true;
    public boolean showGradeColumn = true;
    public boolean showNmValueColumn = true;
    public boolean showValueColumn = true;
    public boolean showInfoColumn = true;
    public boolean showCommentColumn = false;

    static final String LAST_INDEX = "lastIndex";
    public String lastIndex = null;

    static final String GRADE_VALUE_PREFIX = "gradePercentValue_";
    public int gradePercentValues[] = new int[(Issue.GRADE_MAX - Issue.GRADE_NONE) + 1];

    public void load () {
        // Load from our file
        try {
            properties.load (new FileInputStream (propertiesFile));
        }
        catch (IOException ioe) {
            System.out.println ("Notice: Properties file '" + propertiesFile + "' not loaded.");
        }

        // End of line characters?
        String lineSep = properties.getProperty ("line.separator");
        if (lineSep != null) {
            System.setProperty ("line.separator", lineSep);
        }

        // Get last index file
        lastIndex = properties.getProperty (SETTINGS_BASE + LAST_INDEX, "");

        // Get grade percent values
        for (int i=Issue.GRADE_NONE; i<=Issue.GRADE_MAX; i++) {
            try {
                String value = properties.getProperty (SETTINGS_BASE + GRADE_VALUE_PREFIX + i);
                gradePercentValues[i] = (new Integer (value)).intValue();
            }
            catch (NumberFormatException nfe) {
                gradePercentValues[i] = Issue.gradeDefaultPercents[i];
            }
        }

        // Get columns checkboxes
        showDateColumn = properties.getProperty (SETTINGS_BASE + SHOW_DATE_COLUMN, "true").equals ("true");
        showGradeColumn = properties.getProperty (SETTINGS_BASE + SHOW_GRADE_COLUMN, "true").equals ("true");
        showNmValueColumn = properties.getProperty (SETTINGS_BASE + SHOW_NM_VALUE_COLUMN, "false").equals ("true");
        showValueColumn = properties.getProperty (SETTINGS_BASE + SHOW_VALUE_COLUMN, "false").equals ("true");
        showInfoColumn = properties.getProperty (SETTINGS_BASE + SHOW_INFO_COLUMN, "true").equals ("true");
        showCommentColumn = properties.getProperty (SETTINGS_BASE + SHOW_COMMENT_COLUMN, "false").equals ("true");
    }
    public void save () {
        // Set last index file
        properties.setProperty (SETTINGS_BASE + LAST_INDEX, lastIndex);

        // Set grade percent values
        for (int i=Issue.GRADE_NONE; i<=Issue.GRADE_MAX; i++) {
            properties.setProperty (SETTINGS_BASE + GRADE_VALUE_PREFIX + i, String.valueOf (gradePercentValues[i]));
        }

        // Set columns checkboxes
        properties.setProperty (SETTINGS_BASE + SHOW_DATE_COLUMN, showDateColumn ? "true" : "false");
        properties.setProperty (SETTINGS_BASE + SHOW_GRADE_COLUMN, showGradeColumn ? "true" : "false");
        properties.setProperty (SETTINGS_BASE + SHOW_NM_VALUE_COLUMN, showNmValueColumn ? "true" : "false");
        properties.setProperty (SETTINGS_BASE + SHOW_VALUE_COLUMN, showValueColumn ? "true" : "false");
        properties.setProperty (SETTINGS_BASE + SHOW_INFO_COLUMN, showInfoColumn ? "true" : "false");
        properties.setProperty (SETTINGS_BASE + SHOW_COMMENT_COLUMN, showCommentColumn ? "true" : "false");

        // Save to our file
        try {
            properties.store (new FileOutputStream (propertiesFile), propertiesHeader);
        }
        catch (IOException ioe) {
            System.out.println ("Notice: Properties file '" + propertiesFile + "' not saved.");
        }
    }
}