/******************************************************************************
 * Module Name : $RCSfile: CardDialog.java,v $
 *
 * Description : Dialog for managing a card, adding and removing title
 *               references.
 *
 * Updates:
 *   $Log: CardDialog.java,v $
 *   Revision 1.10  2002/07/13 01:06:09  jcouper
 *   job:157
 *   Add +/- for grades.  (Also fixed 147 at the same time).
 *
 *   Revision 1.9  2002/07/08 07:18:38  jcouper
 *   job:140
 *   Order of TitleRefs in card dialog is wrong.
 *
 *   Revision 1.8  2002/07/07 00:54:38  jcouper
 *   job:153
 *   Filenames must be lowercase, and special issues uppercase.
 *
 *   Revision 1.7  2001/10/07 04:42:27  jcouper
 *   141: Now prompts for confirm removing filtered title ref
 *
 *   Revision 1.6  2001/10/07 03:57:35  jcouper
 *   121: Added up/down buttons for titleRef
 *
 *   Revision 1.5  2001/07/11 07:22:31  jcouper
 *   Add confirmation dialog for delete, and filter for titles
 *
 *   Revision 1.4  2001/04/11 08:22:24  jcouper
 *   Add Filter Support
 *
 *   Revision 1.3  2001/04/04 07:34:50  jcouper
 *   Support credits, filters, new columns, etc, etc.  Stable release.
 *
 *   Revision 1.2  2000/08/23 19:34:17  jcouper
 *   Allow renaming of Card.
 *
 *   Revision 1.1  2000/03/18 01:02:45  Jonathan
 *   New file.
 *
 *
 * Copyright Notice:
 * (c) 1999. This source code was developed by Jonathan Couper and friends.
 *            You may use it for any purpose.  No guarantees given or implied.
 *****************************************************************************/
package standup;

import java.util.Enumeration;
import java.util.Vector;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.tree.*;
import standup.parse.*;
import standup.util.*;
import standup.views.*;

public class CardDialog extends JDialog
    implements ActionListener, ListSelectionListener, DocumentListener
{
    // Things on the top (name) panel.
    JPanel namePanel = new JPanel();
    JLabel nameLabel = new JLabel("Name");
    JLimitedTextField nameField = new JLimitedTextField (80, 20);

    // Things on the middle (title) panel.
    JPanel titlePanel = new JPanel();

    JPanel allTitlePanel = new JPanel();
    JPanel allTitleTopPanel = new JPanel();
    JLabel allTitleLabel = new JLabel ("All Titles", SwingConstants.LEFT);
    JCheckBox unusedTitleCheckbox = new JCheckBox ("Unreferenced");
    DefaultListModel allTitleListModel = new DefaultListModel();
    JList allTitleList = new JList(allTitleListModel);
    JScrollPane allTitleScrollPane = new JScrollPane (allTitleList);
    JPanel titleCreatePanel = new JPanel();
    JButton titleCreateButton = new JButton ("Create");
    JAlphaTextField titleCreateField = new JAlphaTextField(20);

    JPanel titleButtonsPanel = new JPanel();
    JPanel moveInnerPanel = new JPanel();

    ImageIcon leftIcon = new ImageIcon (getClass().getResource ("images/left.gif"));
    ImageIcon rightIcon = new ImageIcon (getClass().getResource ("images/right.gif"));
    ImageIcon upIcon = new ImageIcon (getClass().getResource ("images/up.gif"));
    ImageIcon downIcon = new ImageIcon (getClass().getResource ("images/down.gif"));
    ImageIcon editIcon = new ImageIcon (getClass().getResource ("images/edit.gif"));
    JButton titleUpButton = new JButton (upIcon);
    JButton titleDownButton = new JButton (downIcon);
    JButton titleAddButton = new JButton (rightIcon);
    JButton titleRemoveButton = new JButton (leftIcon);
    JButton titleEditButton = new JButton (editIcon);

    JPanel selectedTitlePanel = new JPanel();
    JLabel selectedTitleLabel = new JLabel ("Card Refers To", SwingConstants.LEFT);
    DefaultListModel cardTitleRefListModel = new DefaultListModel();
    JList cardTitleRefList = new JList(cardTitleRefListModel);
    JScrollPane selectedTitleScrollPane = new JScrollPane (cardTitleRefList);
    JPanel titleFilterPanel = new JPanel();
    JLabel titleFilterLabel = new JLabel ("Filter");
    JAlphaTextField titleFilterField = new JAlphaTextField(20);

    // Things on the lower (done) panel.
    JPanel buttonPanel = new JPanel();
    JButton doneButton = new JButton("Done");

    /*
     * We represent the title references on this card.  The title references
     * come from the specified index.
     */
    Index index = null;
    Card card = null;

    /*
     * Creates the dialog panel based on the specified collection.
     */
    public CardDialog (Frame frame, Index newIndex, Card newCard) {
        super (frame, "Card (" + newCard.toString() + ")", true);

        index = newIndex;
        card = newCard;

        // Lowercase filenames
        titleCreateField.setForceLower (true);

        getContentPane().add (namePanel, BorderLayout.NORTH);
        getContentPane().add (titlePanel, BorderLayout.CENTER);
        getContentPane().add (buttonPanel, BorderLayout.SOUTH);

        // Name panel (top)
        namePanel.add (nameLabel);
        namePanel.add (nameField);

        // Title panel (center)
        titlePanel.setLayout (new TripleLayout());
        titlePanel.add (allTitlePanel);
        titlePanel.add (titleButtonsPanel);
        titlePanel.add (selectedTitlePanel);

        allTitleTopPanel.setLayout (new BorderLayout());
        allTitleTopPanel.add (allTitleLabel, BorderLayout.CENTER);
        allTitleTopPanel.add (unusedTitleCheckbox, BorderLayout.EAST);
        unusedTitleCheckbox.setHorizontalAlignment (SwingConstants.RIGHT);

        allTitlePanel.setBorder (BorderFactory.createEmptyBorder (5, 5, 5, 5));
        allTitlePanel.setLayout (new BorderLayout());
        allTitlePanel.add (allTitleTopPanel, BorderLayout.NORTH);
        allTitlePanel.add (allTitleScrollPane, BorderLayout.CENTER);
        allTitlePanel.add (titleFilterPanel, BorderLayout.SOUTH);
        titleFilterPanel.setLayout (new BorderLayout());
        titleFilterPanel.add (titleFilterField, BorderLayout.CENTER);
        titleFilterPanel.add (titleFilterLabel, BorderLayout.WEST);
        allTitleScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        allTitleList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        selectedTitlePanel.setBorder (BorderFactory.createEmptyBorder (5, 5, 5, 5));
        selectedTitlePanel.setLayout (new BorderLayout());
        selectedTitlePanel.add (selectedTitleLabel, BorderLayout.NORTH);
        selectedTitlePanel.add (selectedTitleScrollPane, BorderLayout.CENTER);
        selectedTitlePanel.add (titleCreatePanel, BorderLayout.SOUTH);
        titleCreatePanel.setLayout (new BorderLayout());
        titleCreatePanel.add (titleCreateField, BorderLayout.CENTER);
        titleCreatePanel.add (titleCreateButton, BorderLayout.EAST);
        selectedTitleScrollPane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_ALWAYS);
        cardTitleRefList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        titleButtonsPanel.setBorder (BorderFactory.createEmptyBorder (5, 5, 5, 5));
        titleButtonsPanel.add (moveInnerPanel);
        moveInnerPanel.setLayout (new TopToBottomLayout ());
        moveInnerPanel.add (titleUpButton);
        moveInnerPanel.add (titleAddButton);
        moveInnerPanel.add (titleEditButton);
        moveInnerPanel.add (titleRemoveButton);
        moveInnerPanel.add (titleDownButton);

        /*
         * Set up the title panel, selecting a card will fill this soon.
         */
        buttonPanel.add (doneButton);

        /*
         * Add listeners.
         */
        doneButton.addActionListener (this);
        unusedTitleCheckbox.addActionListener (this);
        allTitleList.addListSelectionListener(this);
        cardTitleRefList.addListSelectionListener(this);
        titleUpButton.addActionListener (this);
        titleAddButton.addActionListener (this);
        titleCreateButton.addActionListener (this);
        titleRemoveButton.addActionListener (this);
        titleEditButton.addActionListener (this);
        titleDownButton.addActionListener (this);
        titleFilterField.getDocument().addDocumentListener(this);
        titleCreateField.getDocument().addDocumentListener(this);
        nameField.getDocument().addDocumentListener(this);

        /*
         * Enable/disable and configure.
         */
        titleCreateButton.setEnabled (false);
        titleAddButton.setEnabled (false);
        titleRemoveButton.setEnabled (false);
        titleEditButton.setEnabled (false);

        /*
         * Fill lists with the titles.
         */
        nameField.setText (card.getName());
        fillCardTitles ();

        /*
         * Resize and position ourselves.
         */
        pack();

        Dimension pref = this.getPreferredSize();
        setSize(pref.width, pref.height);

        Dimension meSize = getSize();
        Dimension momSize = frame.getSize();
        Point momPos = frame.getLocationOnScreen();
        int x = momPos.x + (momSize.width / 2) - (meSize.width / 2);
        int y = momPos.y + (momSize.height / 2) - (meSize.height / 2);

        this.setLocation ((x > 0) ? x : 0, (y > 0) ? y : 0);
    }

    /*
     * Fill the title listboxes.  The check for titleReferences is very
     * inefficient.  Sorry.
     */
    public void fillCardTitles () {
        Vector allTitles = index.getAllTitles ();
        boolean unusedOnly = unusedTitleCheckbox.isSelected ();

        cardTitleRefListModel.clear();
        allTitleListModel.clear();

        /*
         * Loop through all the titles we have.  This will thrash the
         * disk pretty hard the first time we do this.  These go on
         * the left hand list (except for those on the card).
         */
        for (Enumeration e = allTitles.elements() ; e.hasMoreElements() ;) {
            Title title = (Title) e.nextElement ();
            TitleRef titleRef = card.cardReferenceTo (title);
            String filterString = titleFilterField.getText ().toLowerCase ();

            // Already on the current card - don't make it available
            if (titleRef != null) {

            // A checkbox makes us hide those that are already on cards
            } else if (unusedOnly && index.containsReferenceTo (title)) {
                /* Don't want this one */

            // No filter?
            } else if (filterString.equals("")) {
                allTitleListModel.addElement (title);

            // Filter, only list specified titles
            } else if (title.toString().toLowerCase().indexOf (filterString) != -1) {
                allTitleListModel.addElement (title);
            }
        }

        /*
         * Now get the titleRefs from our card for the right hand side.
         */
        for (Enumeration e = card.elements() ; e.hasMoreElements() ;) {
            TitleRef titleRef = (TitleRef) e.nextElement ();
            cardTitleRefListModel.addElement (titleRef);
        }

        enableTitleTransferButtons ();
    }

    /*
     * Figure out which title transfer buttons are valid right now.
     */
    public void enableTitleTransferButtons () {
        int allIdx = allTitleList.getSelectedIndex();
        int cardIdx = cardTitleRefList.getSelectedIndex();
        int cardSize = cardTitleRefList.getModel().getSize();

        boolean allSelected = (allIdx >= 0);
        boolean cardSelected = (cardIdx >= 0);

        if (card == null) {
            titleUpButton.setEnabled (false);
            titleAddButton.setEnabled (false);
            titleRemoveButton.setEnabled (false);
            titleDownButton.setEnabled (false);
            titleEditButton.setEnabled (allSelected);

        } else {
            titleUpButton.setEnabled (cardSelected && (cardIdx > 0));
            titleAddButton.setEnabled (allSelected);
            titleRemoveButton.setEnabled (cardSelected);
            titleDownButton.setEnabled (cardSelected && (cardIdx >= 0) && (cardIdx < (cardSize - 1)));
            titleEditButton.setEnabled (allSelected || cardSelected);
        }
    }

    /*
     **************************************************************************
     * LISTENERS BELOW HERE
     **************************************************************************
     */
    /*
     **************************************************************************
     * List listeners.
     */
    public void valueChanged(ListSelectionEvent e) {
        /*
         * When the all title list is selected, clear the other list, and enable
         * the move to selected button.
         */
        if (e.getSource() == allTitleList) {
            boolean selected = (allTitleList.getSelectedIndex() >= 0);
            if (selected) {
                cardTitleRefList.clearSelection();
            }

        } else if (e.getSource() == cardTitleRefList) {
            boolean selected = (cardTitleRefList.getSelectedIndex() >= 0);
            if (selected) {
                allTitleList.clearSelection();
            }
        }
        enableTitleTransferButtons();
    }

    protected void exchangeCardRefs (int posFrom, int posTo) {
        TitleRef refFrom = (TitleRef) cardTitleRefListModel.get (posFrom);
        TitleRef refTo = (TitleRef) cardTitleRefListModel.get (posTo);
        cardTitleRefListModel.set (posFrom, refTo);
        cardTitleRefListModel.set (posTo, refFrom);
        card.set (posFrom, refTo);
        card.set (posTo, refFrom);

        // Keep the item being moved as selected.
        cardTitleRefList.setSelectedValue (refFrom, true);
    }

    /*
     **************************************************************************
     * Act on our button actions.
     */
    public void actionPerformed(ActionEvent e) {
        Object source = e.getSource();

        // Done enough damage, close the screen.
        if (source == doneButton) {
            this.setVisible (false);

        // Change checkbox filter state.
        } else if (source == unusedTitleCheckbox) {
            fillCardTitles ();

        // Makin' a new title.  We don't care what card it belongs to or anything.
        } else if (source == titleCreateButton) {
            /*
             * Add the new title to the index.  Fail if it already exists.  That
             * shouldn't be possible, but it doesn't hurt to check.
             *
             * This has actually created our new title.
             */
            String proposedTitleKey = titleCreateField.getText();
            Title newTitle = index.newTitleForKeyname (proposedTitleKey);
            if (newTitle == null) {
                return;
            }

            // OK, our button has done its work for now.  Disable it.
            titleCreateButton.setEnabled (false);

            // Add an issue #1 to the title list.
            newTitle.addElement (new IssueRange (new Issue ("1")));
            newTitle.flagTemplateDataModified();

            // Put the title on this card
            TitleRef titleRef = new TitleRef (index, newTitle);
            card.addElement (titleRef);

            // Add it to the list and select it (scroll to it)
            cardTitleRefListModel.addElement (titleRef);
            cardTitleRefList.setSelectedValue (titleRef, true);

            // Edit the new title.
            TitleDialog titleDialog = new TitleDialog (this, newTitle);
            titleDialog.setVisible (true);

            // Borrow its name, if it's the only title
            if (cardTitleRefListModel.getSize() == 1) {
                nameField.setText (titleRef.toString());
            }

            // And refresh the list, since we've added to it.
            cardTitleRefList.repaint();

        // Add reference to selected title
        } else if (source == titleAddButton) {
            Title title = (Title) allTitleList.getSelectedValue();
            if (title != null) {
                TitleRef titleRef = new TitleRef (index, title);
                card.addElement (titleRef);
                allTitleListModel.removeElement (title);
                cardTitleRefListModel.addElement (titleRef);
                cardTitleRefList.setSelectedValue (titleRef, true);
                index.setModified (true);
                if (cardTitleRefListModel.getSize() == 1) {
                    nameField.setText (titleRef.toString());
                }
            }

        // Remove reference to selected title
        } else if (source == titleRemoveButton) {
            TitleRef titleRef = (TitleRef) cardTitleRefList.getSelectedValue();
            if (titleRef != null) {
                // Check before removing title references with filter details.
                if (titleRef.isFiltered ()) {
                    if (! Application.getMainFrame().getConfirm ("Remove Filtered TitleRef '" + titleRef + "'?", "Delete Confirmation")) {
                        return;
                    }
                }

                Title title = titleRef.getTitle();
                card.removeElement (titleRef);
                cardTitleRefListModel.removeElement (titleRef);
                allTitleListModel.addElement (title);
                allTitleList.setSelectedValue (title, true);
                index.setModified (true);
            }

        // Shuffle Up/Down
        } else if (source == titleUpButton) {
            int cardIdx = cardTitleRefList.getSelectedIndex();
            exchangeCardRefs (cardIdx, cardIdx - 1);
            index.setModified (true);

        } else if (source == titleDownButton) {
            int cardIdx = cardTitleRefList.getSelectedIndex();
            exchangeCardRefs (cardIdx, cardIdx + 1);
            index.setModified (true);

        // Edit title or title reference
        } else if (source == titleEditButton) {
            // Is it a title from the all titles list?
            Title title = (Title) allTitleList.getSelectedValue();
            TitleRef titleRef = null;

            if (title == null) {
                // Maybe it's a title reference
                titleRef = (TitleRef) cardTitleRefList.getSelectedValue();
                title = titleRef.getTitle();
            }
            if (title == null) {
                return;
            }

            // Either way, edit the title/ref
            if (titleRef != null) {
                TitleDialog titleDialog = new TitleDialog (this, titleRef);
                titleDialog.setVisible (true);

            } else {
                TitleDialog titleDialog = new TitleDialog (this, title);
                titleDialog.setVisible (true);
            }

            // And refresh the lists, since the name might have changed
            allTitleList.repaint();
            cardTitleRefList.repaint();
        }
    }

    /*
     **************************************************************************
     * Act on document changes.
     */
    public void changedUpdate (DocumentEvent e) {
        documentUpdate (e);
    }
    public void insertUpdate (DocumentEvent e) {
        documentUpdate (e);
    }
    public void removeUpdate (DocumentEvent e) {
        documentUpdate (e);
    }
    public void documentUpdate (DocumentEvent e) {
        Document source = e.getDocument();
        if (source == titleCreateField.getDocument()) {
            String proposedTitleKey = titleCreateField.getText();
            if (proposedTitleKey.equals("")) {
                titleCreateButton.setEnabled (false);

            } else {
                titleCreateButton.setEnabled (index.existingTitleForKeyname(proposedTitleKey) == null);
            }

        } else if (source == nameField.getDocument()) {
            if (!card.getName().equals (nameField.getText())) {
                card.setName (nameField.getText());
                index.setModified (true);
            }

        } else if (source == titleFilterField.getDocument()) {
            fillCardTitles ();
        }
    }

    /*
     **************************************************************************
     * INNER LAYOUT CLASS
     **************************************************************************
     */
    class TripleLayout
    implements LayoutManager2
    {
        public TripleLayout()
        {
        }

        public void addLayoutComponent(String name, Component comp) {}
        public void removeLayoutComponent(Component comp) {}

        public Dimension preferredLayoutSize(Container parent)
        {
            int numComps = parent.getComponentCount();
            if (numComps != 3) {
                return null;
            }

            int height = 0;
            int width = 0;
            for (int a = 0; (a < numComps); a++) {
                Component c;
                Dimension d;

                c = parent.getComponent(a);
                d = c.getPreferredSize();

                // We set our preferred height to the max of the three children.
                if (d.height > height) {
                    height = d.height;
                }

                // Preferred width is the sum of all three.
                width = width + d.width;
            }
            Dimension retVal = new Dimension (width, height);

            return retVal;
        }

        public Dimension minimumLayoutSize(Container parent)
        {
            return preferredLayoutSize (parent);
        }

        public void layoutContainer(Container parent)
        {
            int numComps = parent.getComponentCount();
            if (numComps != 3) {
                return;
            }

            // Get parent size.
            Dimension ps = parent.getSize();
            Insets pi = parent.getInsets();
            ps.width = ps.width - (pi.left + pi.right);
            ps.height = ps.height - (pi.top + pi.bottom);

            // Figure what middle component wants.
            Component c1 = parent.getComponent(1);
            Dimension c1d = c1.getPreferredSize();

            int width1 = c1d.width;
            c1.setSize (width1, ps.height);

            // Half width on the left.
            Component c0 = parent.getComponent(0);
            int width0 = (ps.width - width1) / 2;
            c0.setSize (width0, ps.height);

            // Remainder on the right.
            Component c2 = parent.getComponent(2);
            int width2 = ps.width - (width0 + width1);
            c2.setSize (width2, ps.height);

            c0.setLocation (pi.left, pi.top);
            c1.setLocation (pi.left + width0, pi.top + (ps.height - c1d.height) / 2);
            c2.setLocation (pi.left + width0 + width1, pi.top);
        }


        public void addLayoutComponent(Component c, Object constraints) {}
        public Dimension maximumLayoutSize(Container target)
        {
            return null;
        }
        public float getLayoutAlignmentX(Container target)
        {
            return 0;
        }
        public float getLayoutAlignmentY(Container target)
        {
            return 0;
        }
        public void invalidateLayout(Container target) {}
    }
}