/******************************************************************************
 * Module Name : $RCSfile: EditDialog.java,v $
 *
 * Description : Little Dialog for editing comic data (all or partial).
 *
 * Updates:
 *   $Log: EditDialog.java,v $
 *   Revision 1.8  2002/07/14 03:48:29  jcouper
 *   job:159
 *   Option to over-ride the automatic value.
 *
 *   Revision 1.7  2002/07/13 01:06:10  jcouper
 *   job:157
 *   Add +/- for grades.  (Also fixed 147 at the same time).
 *
 *   Revision 1.6  2002/07/07 01:16:54  jcouper
 *   job:155
 *   Forward and Back keys in edit issue dialog.
 *
 *   Revision 1.5  2002/07/07 00:57:02  jcouper
 *   job:150
 *   Credits cut-and-paste use system clipboard.
 *
 *   Revision 1.4  2001/07/29 03:06:01  jcouper
 *   job:137
 *   Add cut and paste for credits, plus default values.
 *
 *   Revision 1.3  2001/07/29 00:47:27  jcouper
 *   job:137
 *   Added support for credits, and re-vamped the EditDialog.
 *
 *   Revision 1.2  2000/01/10 02:22:20  Jonathan
 *   v1.0.0
 *
 *   Revision 1.1  2000/01/09 00:11:55  Jonathan
 *   Edit Dialog, Preferences
 *
 *   Revision 1.10  2000/01/06 23:46:15  Jonathan
 *   Standup v0.9.3 - Minor Fixes, and report functionality.
 *
 *   Revision 1.9  1999/11/27 10:31:29  Jonathan
 *   Interface tweaks.  Reporting on the menu.  Fooling around with HTML
 *   about box.
 *
 *   Revision 1.8  1999/10/25 08:33:07  Jonathan
 *   v0_9_2
 *
 *   Revision 1.7  1999/10/24 22:37:54  Jonathan
 *   Proper CVS headers.  Fix A-018/19/20
 *
 *
 * Copyright Notice:
 * (c) 1999. This source code was developed by Jonathan Couper and friends.
 *            You may use it for any purpose.  No guarantees given or implied.
 *****************************************************************************/
package standup;

import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.util.*;
import javax.swing.event.*;
import javax.swing.*;
import javax.swing.text.*;
import javax.swing.border.*;
import standup.parse.*;
import standup.util.*;
import standup.views.*;

public class EditDialog extends JDialog
    implements ActionListener, ItemListener, DocumentListener,
                FocusListener, KeyListener, ClipboardOwner
{
    // Just to be consistent.
    String notSelectedString = "No Issue Selected";

    // We use this a couple of times.
    Border emptyBorder = BorderFactory.createEmptyBorder (5, 5, 5, 5);

    JPanel mainPanel = new JPanel();

    // The bottom button panel.
    JPanel buttonPanel = new JPanel();
    JButton closeButton = new JButton ("Close");
    ImageIcon upIcon = new ImageIcon (getClass().getResource ("images/up.gif"));
    ImageIcon downIcon = new ImageIcon (getClass().getResource ("images/down.gif"));
    JButton upButton = new JButton (upIcon);
    JButton downButton = new JButton (downIcon);

    // This is our template controls.
    JPanel templatePanel = new JPanel ();
    BorderLayout templateLayout = new BorderLayout ();
    JTextField infoTextField = new JTextField (40);

    // This goes on the template panel
    JPanel templateDetailPanel = new JPanel ();
    TopToBottomLayout templateDetailLayout = new TopToBottomLayout ();
    ValueComponent nmValueComponent = new ValueComponent ();
    CoverDateComponent coverDateComponent = new CoverDateComponent ();

    // This goes on the template panel too
    JPanel creditPanel = new JPanel ();

    JPanel creditInnerPanel = new JPanel ();
    TopToBottomLayout creditInnerLayout = new TopToBottomLayout ();
    JTextField creditWhatFields[] = new JTextField[Issue.MAX_CREDITS];
    JLabel creditEqualsLabels[] = new JLabel[Issue.MAX_CREDITS];
    JTextField creditWhoFields[] = new JTextField[Issue.MAX_CREDITS];

    JPanel creditButtonPanel = new JPanel ();
    JButton creditCopyButton = new JButton ("Copy");
    JButton creditPasteButton = new JButton ("Paste");

    // These are our user controls.
    JPanel userPanel = new JPanel();
    BorderLayout userLayout = new BorderLayout ();
    JTextField commentTextField = new JTextField (32);

    // User detail panel
    JPanel userDetailPanel = new JPanel ();
    TopToBottomLayout userDetailLayout = new TopToBottomLayout ();
    JComboBox gradeCombo = new JComboBox();
    JComboBox gradeModCombo = new JComboBox();
    ValueComponent gradeValueComponent = new ValueComponent ();
    JCheckBox gradeAutoCheckbox = new JCheckBox ("Auto");

    // A clipboard for saving credits.
    Credit creditClipboard[] = new Credit[Issue.MAX_CREDITS];

    /*
     * This points to what we're editing.
     */
    MainFrame frame = null;
    boolean active = false;

    Issue editedIssue = null;
    TitleRef editedTitleRef = null;

    /*
     * Constructor?  Not really.
     */
    public EditDialog (MainFrame inFrame) {
        frame = inFrame;

        getContentPane().add (mainPanel, BorderLayout.CENTER);
        getContentPane().add (buttonPanel, BorderLayout.SOUTH);

        mainPanel.setLayout (new BorderLayout());

        mainPanel.add (templatePanel, BorderLayout.NORTH);
        mainPanel.add (userPanel, BorderLayout.CENTER);

        buttonPanel.add (upButton);
        buttonPanel.add (downButton);
        buttonPanel.add (closeButton);

        templateDetailPanel.setBorder (new CompoundBorder(BorderFactory.createTitledBorder("Details"), emptyBorder));
        templateDetailLayout.setAlignment (TopToBottomLayout.ALIGN_LEFT);
        templateDetailLayout.setColumns (1, 1);
        templateDetailLayout.setVgap (5);
        templateDetailPanel.setLayout (templateDetailLayout);
        templateDetailPanel.add (coverDateComponent);
        templateDetailPanel.add (new JLabel ("NM Value"));
        templateDetailPanel.add (nmValueComponent);

        // This is the inner credit panel with the text fields
        creditInnerLayout.setAlignment (LeftToRightLayout.ALIGN_STRETCH);
        creditInnerLayout.setColumns (3, 3);
        creditInnerPanel.setLayout (creditInnerLayout);

        for (int i=0; i < Issue.MAX_CREDITS; i++) {
            creditWhatFields[i] = new JTextField(12);
            creditEqualsLabels[i] = new JLabel (" = ");
            creditWhoFields[i] = new JTextField(20);

            creditWhatFields[i].getDocument().addDocumentListener (this);
            creditWhoFields[i].getDocument().addDocumentListener (this);

            creditInnerPanel.add (creditWhatFields[i]);
            creditInnerPanel.add (creditEqualsLabels[i]);
            creditInnerPanel.add (creditWhoFields[i]);
        }

        // This is a credit button panel inside the main credit panel
        creditButtonPanel.add (creditCopyButton);
        creditButtonPanel.add (creditPasteButton);
        creditCopyButton.addActionListener (this);
        creditPasteButton.addActionListener (this);

        // This is the main credit sub-panel
        creditPanel.setBorder (new CompoundBorder(BorderFactory.createTitledBorder("Credits"), emptyBorder));
        creditPanel.setLayout (new BorderLayout ());
        creditPanel.add (creditInnerPanel, BorderLayout.CENTER);
        creditPanel.add (creditButtonPanel, BorderLayout.SOUTH);

        // This is the main template panel
        templatePanel.setBorder (new CompoundBorder(BorderFactory.createTitledBorder("Template Data"), emptyBorder));
        templateLayout.setHgap (5);
        templatePanel.setLayout (templateLayout);
        templatePanel.add (infoTextField, BorderLayout.NORTH);
        templatePanel.add (templateDetailPanel, BorderLayout.WEST);
        templatePanel.add (creditPanel, BorderLayout.CENTER);

        // User Detail Panel
        userDetailLayout.setAlignment (TopToBottomLayout.ALIGN_STRETCH);
        userDetailLayout.setColumns (6, 3);
        userDetailLayout.setVAlignment (TopToBottomLayout.ALIGN_VTOP);
        userDetailLayout.setHgap (5);
        userDetailPanel.setLayout (userDetailLayout);
        userDetailPanel.add (gradeCombo);
        userDetailPanel.add (gradeModCombo);
        userDetailPanel.add (new JLabel (" "));
        userDetailPanel.add (new JLabel ("Grade Value"));
        userDetailPanel.add (gradeValueComponent);
        userDetailPanel.add (gradeAutoCheckbox);

        // This is the main user panel.
        userPanel.setBorder (new CompoundBorder(BorderFactory.createTitledBorder("User Data"), emptyBorder));
        userLayout.setVgap (5);
        userPanel.setLayout (userLayout);
        userPanel.add (commentTextField, BorderLayout.NORTH);
        userPanel.add (userDetailPanel, BorderLayout.CENTER);

        /*
         * Set parameters.
         */
        for (int i = 0; i <= Issue.GRADE_MAX; i++) {
            String text = Issue.getNameForGrade (i);
            gradeCombo.addItem (text);
        }
        gradeModCombo.addItem (" -- ");
        gradeModCombo.addItem (" - ");
        gradeModCombo.addItem (" ");
        gradeModCombo.addItem (" + ");
        gradeModCombo.addItem (" ++ ");

        /*
         * Add listeners.
         */
        upButton.addActionListener (this);
        downButton.addActionListener (this);
        closeButton.addActionListener (this);
        nmValueComponent.addActionListener (this);
        coverDateComponent.addActionListener (this);
        gradeCombo.addItemListener (this);
        gradeModCombo.addItemListener (this);
        gradeValueComponent.addActionListener (this);
        gradeAutoCheckbox.addActionListener (this);
        commentTextField.getDocument().addDocumentListener (this);
        infoTextField.getDocument().addDocumentListener (this);
        this.addKeyListener (this);

        setEmpty ();
        this.validate ();
    }


    public void setVisible (boolean state) {
        /*
         * Resize and position ourselves.
         */
        if (state) {
            pack ();

            Dimension pref = this.getPreferredSize();
            setSize(pref.width, pref.height);

            Dimension meSize = getSize();
            Dimension momSize = frame.getSize();
            Point momPos = frame.getLocationOnScreen();
            int x = momPos.x + (momSize.width / 2) - (meSize.width / 2);
            int y = momPos.y + (momSize.height / 2) - (meSize.height / 2);

            this.setLocation ((x > 0) ? x : 0, (y > 0) ? y : 0);
        }
        super.setVisible (state);
    }

    /*
     * We're editing a row of a table.  We can figure the rest out for ourselves.
     * Might as well store the issue and tableRef's for later.  Quicker that way.
     */
    public void editNow (TitleRef titleRef, Issue issue) {
        // Flag that we're not ready.
        active = false;

        editedIssue = issue;
        editedTitleRef = titleRef;

        // Template Data
        setTitle (editedTitleRef.getAnyName() + " #" + editedIssue.getCode());
        infoTextField.setText (editedIssue.getInfo());
        infoTextField.setEnabled (true);
        nmValueComponent.setValue (editedIssue.getNmValue());
        nmValueComponent.setEnabled (true);
        coverDateComponent.setCoverDate (editedIssue.getCoverDate());
        coverDateComponent.setEnabled (true);

        // User Data
        gradeCombo.setSelectedIndex (editedIssue.getGrade());
        gradeCombo.setEnabled (true);
        gradeModCombo.setSelectedIndex (editedIssue.getGradeMod() + 2);
        gradeModCombo.setEnabled (editedIssue.isGradeModUsed());
        commentTextField.setText (editedIssue.getComment());
        commentTextField.setEnabled (true);
        gradeAutoCheckbox.setSelected (! editedIssue.isGradeValueOverride());
        gradeAutoCheckbox.setEnabled (true);
        gradeValueComponent.setValue (editedIssue.getGradeValue());
        gradeValueComponent.setEnabled (editedIssue.isGradeValueOverride());

        boolean haveCredits = false;
        for (int i=0; i < Issue.MAX_CREDITS; i++) {
            Credit credit = editedIssue.getCredit (i);
            if (credit == null) {
                creditWhatFields[i].setText ("");
                creditWhoFields[i].setText ("");

            } else {
                creditWhatFields[i].setText (credit.what);
                creditWhoFields[i].setText (credit.who);
                haveCredits = true;
            }
        }
        if (! haveCredits) {
            creditWhatFields[0].setText ("Editor");
            creditWhatFields[1].setText ("Writer");
            creditWhatFields[2].setText ("Penciler");
            creditWhatFields[3].setText ("Inker");
        }

        // OK, now let's go.
        active = true;
        this.repaint();
    }

    /*
     * Clear self.  We're not editing anything now.
     */
    public void setEmpty () {
        active = false;

        editedIssue = null;
        editedTitleRef = null;

        // Template Data
        setTitle (notSelectedString);
        infoTextField.setText ("");
        infoTextField.setEnabled (false);
        nmValueComponent.setValue (0.0);
        nmValueComponent.setEnabled (false);
        coverDateComponent.setCoverDate (null);
        coverDateComponent.setEnabled (true);

        // User Data
        commentTextField.setText ("");
        commentTextField.setEnabled (false);
        gradeCombo.setSelectedIndex (Issue.GRADE_NONE);
        gradeCombo.setEnabled (false);
        gradeValueComponent.setValue (0.0);
        gradeValueComponent.setEnabled (false);
        gradeAutoCheckbox.setSelected (true);
        gradeAutoCheckbox.setEnabled (false);
    }

    /*
     * Grade chosen?
     */
    public void itemStateChanged(ItemEvent e) {
        if (!active) return;

        if (e.getStateChange() != ItemEvent.SELECTED) {
            return;
        }

        /*
         * Grade?  Better let the table know we've made a change.
         */
        if (e.getSource() == gradeCombo) {
            int newGrade = gradeCombo.getSelectedIndex();

            editedIssue.setGrade (newGrade);
            editedTitleRef.getTitle().flagUserDataModified();

            if (editedIssue.isGradeModUsed()) {
                gradeModCombo.setEnabled (true);

            } else {
                gradeModCombo.setSelectedIndex (2);
                gradeModCombo.setEnabled (false);
            }

            if (gradeAutoCheckbox.isSelected ()) {
                gradeValueComponent.setValue (editedIssue.getAutoGradeValue());
            }

            frame.fireUpdateEditedRow ();

        /*
         * Grade Mod?  Better let the table know we've made a change.
         */
        } else if (e.getSource() == gradeModCombo) {
            int newGradeMod = gradeModCombo.getSelectedIndex() - 2;

            editedIssue.setGradeMod (newGradeMod);
            editedTitleRef.getTitle().flagUserDataModified();

            if (gradeAutoCheckbox.isSelected ()) {
                gradeValueComponent.setValue (editedIssue.getAutoGradeValue());
            }

            frame.fireUpdateEditedRow ();
        }
    }

    /*
     * These come from our buttons, and we also get notified of some
     * of the main menu buttons.
     */
    public void actionPerformed(ActionEvent e) {
        if (!active) return;

        Object source = e.getSource();

        /*
         * The JMenuItem buttons are from the MainFrame main menu.
         */
        if (source instanceof JMenuItem) {
            String command = e.getActionCommand();
            editedIssue.setGradeAndMod (command);
            editedTitleRef.getTitle().flagUserDataModified();
            frame.fireUpdateEditedRow();

        } else if (source == upButton) {
            frame.selectDeltaRow (-1);

        } else if (source == downButton) {
            frame.selectDeltaRow (1);

        } else if (source == closeButton) {
            setVisible (false);

        } else if (source == nmValueComponent) {
            double value = nmValueComponent.getValue();
            editedIssue.setNmValue (value);
            editedTitleRef.getTitle().flagTemplateDataModified();

            if (gradeAutoCheckbox.isSelected ()) {
                gradeValueComponent.setValue (editedIssue.getGradeValue());
            }

            frame.fireUpdateEditedRow();

        } else if (source == gradeValueComponent) {
            if (gradeAutoCheckbox.isSelected ()) {
                return;
            }

            double value = gradeValueComponent.getValue();
            editedIssue.setGradeValue (value);
            editedTitleRef.getTitle().flagUserDataModified();

            frame.fireUpdateEditedRow();

        } else if (source == coverDateComponent) {
            editedTitleRef.getTitle().flagTemplateDataModified();
            editedTitleRef.getTitle().computeDerivedDates();
            frame.fireUpdateAllRows();

        /*
         * Copy all credits as multi-line text to system clipboard.
         */
        } else if (source == creditCopyButton) {
            String creditsAsText = "";

            for (int i=0; i < Issue.MAX_CREDITS; i++) {
                Credit credit = editedIssue.getCredit (i);
                if (credit != null) {
                    creditsAsText = creditsAsText + credit.what + "=" + credit.who + "\n";
                }
            }

            getToolkit().getSystemClipboard().setContents (
                new StringSelection (creditsAsText),
                this);

        /*
         * Paste all credits as multi-line text from system clipboard.
         */
        } else if (source == creditPasteButton) {
            Transferable transferable = getToolkit().getSystemClipboard().getContents (null);

            // Check we have some contents of the right kind
            if ((transferable == null) || !(transferable instanceof StringSelection)) {
                return;
            }

            // Turn our contents back into a string
            StringSelection creditsStringSelection = (StringSelection) transferable;

            String creditsAsText = "";
            try {
                creditsAsText = (String) transferable.getTransferData (DataFlavor.stringFlavor);
            }
            catch (Exception ex) {
                return;
            }

            // Break up the string line by line
            MyStringTokenizer tokenizer = new MyStringTokenizer (creditsAsText.trim(), "\n");

            String token = null;
            int i = 0;

            while (tokenizer.hasMoreTokens() && (i < Issue.MAX_CREDITS)) {
                try {
                    token = tokenizer.nextToken();
                }
                catch (NoSuchElementException nse) {
                    break;
                }

                if (token.indexOf("=") > -1) {
                    creditWhatFields[i].setText (token.substring (0, token.indexOf("=")));
                    creditWhoFields[i].setText (token.substring (token.indexOf("=") + 1));

                } else {
                    creditWhatFields[i].setText (token);
                    creditWhoFields[i].setText ("");
                }

                i++;
            }

            for (i = i; i < Issue.MAX_CREDITS; i++) {
                creditWhatFields[i].setText ("");
                creditWhoFields[i].setText ("");
            }

            editedTitleRef.getTitle().flagTemplateDataModified();

        /*
         * Auto-checkbox?
         */
        } else if (e.getSource() == gradeAutoCheckbox) {

            // Go to auto, clear any override and use the auto default
            if (gradeAutoCheckbox.isSelected ()) {
                editedIssue.setGradeValue (0.0);
                gradeValueComponent.setValue (editedIssue.getAutoGradeValue());
                gradeValueComponent.setEnabled (false);

            // No more auto, explicitly set the grade
            } else {
                editedIssue.setGradeValue (gradeValueComponent.getValue());
                gradeValueComponent.setEnabled (true);
            }
        }
    }

    /*
     * Act on document changes.
     */
    public void changedUpdate (DocumentEvent e) {
        documentUpdate (e);
    }
    public void insertUpdate (DocumentEvent e) {
        documentUpdate (e);
    }
    public void removeUpdate (DocumentEvent e) {
        documentUpdate (e);
    }
    public void documentUpdate (DocumentEvent e) {
        if (!active) return;

        Document source = e.getDocument();
        if (source == commentTextField.getDocument()) {
            editedIssue.setComment (commentTextField.getText().replace ('\n', ' '));
            editedTitleRef.getTitle().flagUserDataModified();
            frame.fireUpdateEditedRow();

        } else if (source == infoTextField.getDocument()) {
            editedIssue.setInfo (infoTextField.getText().replace ('\n', ' '));
            editedTitleRef.getTitle().flagTemplateDataModified();
            frame.fireUpdateEditedRow();

        } else {
            for (int i=0; i < Issue.MAX_CREDITS; i++) {
                if ((source == creditWhatFields[i].getDocument()) ||
                    (source == creditWhoFields[i].getDocument())) {

                    editedIssue.setCredit (creditWhatFields[i].getText (),
                        creditWhoFields[i].getText (), i);

                    editedTitleRef.getTitle().flagTemplateDataModified();
                }
            }
        }
    }

    /*
     **************************************************************************
     * Key listeners
     */
    public void keyPressed(KeyEvent e) {
        // Ctrl-F is the down-row button
        if ((e.getKeyCode() == KeyEvent.VK_F) && ((e.getModifiers() & KeyEvent.CTRL_MASK) != 0)) {
            frame.selectDeltaRow (1);

        // Ctrl-B is the up-row button
        } else if ((e.getKeyCode() == KeyEvent.VK_B) && ((e.getModifiers() & KeyEvent.CTRL_MASK) != 0)) {
            frame.selectDeltaRow (-1);
        }
    }
    public void keyReleased(KeyEvent e) {}
    public void keyTyped(KeyEvent e) {}

    /*
     **************************************************************************
     * Focus listeners
     */
    public void focusGained(FocusEvent e) {}
    public void focusLost(FocusEvent e) {}

    /*
     **************************************************************************
     * Clipboard listeners
     */
    public void lostOwnership (Clipboard clipboard, Transferable contents) {}
}
