/******************************************************************************
 * Module Name : $RCSfile: IndexDialog.java,v $
 *
 * Description : Dialog for managing an index, adding and removing collections
 *			 	  and cards.
 *
 * Updates:
 *   $Log: IndexDialog.java,v $
 *   Revision 1.7  2000/03/18 00:46:04  Jonathan
 *   Big changes in the way the IndexDialog works, including a new CardDialog.
 *   New preferences for column display.  Permit renaming of title files.
 *
 *   Revision 1.6  2000/01/06 23:46:16  Jonathan
 *   Standup v0.9.3 - Minor Fixes, and report functionality.
 *
 *   Revision 1.5  1999/10/25 08:33:07  Jonathan
 *   v0_9_2
 *
 *   Revision 1.4  1999/10/24 22:37:54  Jonathan
 *   Proper CVS headers.  Fix A-018/19/20
 *
 *
 * Copyright Notice:
 * (c) 1999. This source code was developed by Jonathan Couper and friends. 
 * 	      	  You may use it for any purpose.  No guarantees given or implied.
 *****************************************************************************/ 
package standup;

import java.util.Enumeration;
import java.util.Vector;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.tree.*;
import standup.parse.*;
import standup.util.*;
import standup.views.*;

public class IndexDialog extends JDialog 
	implements ActionListener, TreeSelectionListener, 
			   TreeModelListener, MouseListener
{
	// Things on the upper (index) panel.
	JPanel topPanel = new JPanel();
	DefaultTreeModel indexModel = new DefaultTreeModel (new DefaultMutableTreeNode());
	JTree indexTree = new JTree(indexModel);
	JScrollPane indexPane = new JScrollPane(indexTree);

	// Things on the lower (done) panel.
	JPanel buttonPanel = new JPanel();
	ImageIcon upIcon = new ImageIcon (getClass().getResource ("images/up.gif"));
	ImageIcon addAboveIcon = new ImageIcon (getClass().getResource ("images/add_above.gif"));
	ImageIcon deleteIcon = new ImageIcon (getClass().getResource ("images/delete.gif"));
	ImageIcon addBelowIcon = new ImageIcon (getClass().getResource ("images/add_below.gif"));
	ImageIcon downIcon = new ImageIcon (getClass().getResource ("images/down.gif"));
	JButton upButton = new JButton (upIcon);
	JButton addAboveButton = new JButton (addAboveIcon);
	JButton deleteButton = new JButton (deleteIcon);
	JButton addBelowButton = new JButton (addBelowIcon);
	JButton downButton = new JButton (downIcon);
	JButton doneButton = new JButton("Done");
	
	// Popup menu for cards.
	JPopupMenu cardPopup = new JPopupMenu ("Card Menu");
	JMenuItem cardEditItem = new JMenuItem ("Edit");
	JMenuItem cardRenameItem = new JMenuItem ("Rename");
	JMenuItem cardDeleteItem = new JMenuItem ("Delete");
	// SEP
	JMenuItem cardMoveUpItem = new JMenuItem ("Move Up");
	JMenuItem cardMoveDownItem = new JMenuItem ("Move Down");
	
	// Popup menu for collections.
	JPopupMenu collectionPopup = new JPopupMenu ("Collection Menu");
	JMenuItem collectionRenameItem = new JMenuItem ("Rename");
	JMenuItem collectionDeleteItem = new JMenuItem ("Delete");
	// SEP
	JMenuItem collectionSortItem = new JMenuItem ("Sort A..Z");
	JMenuItem collectionMoveUpItem = new JMenuItem ("Move Up");
	JMenuItem collectionMoveDownItem = new JMenuItem ("Move Down");

	// Popup menu for indexes.
	JPopupMenu indexPopup = new JPopupMenu ("Index Menu");
	JMenuItem indexSortItem = new JMenuItem ("Sort A..Z");
	
	/*
	 * We represent the title references on this card.  The title references
	 * come from the specified index.
	 */
	Frame frame = null;
	Index index = null;
	
	/*
	 * Creates the dialog panel based on the specified collection.
	 */
	public IndexDialog (Frame newFrame, Index newIndex) {
		super (newFrame, newIndex.toString(), true);

		frame = newFrame;
		index = newIndex;
		
		getContentPane().add (topPanel, BorderLayout.CENTER);
		getContentPane().add (buttonPanel, BorderLayout.SOUTH);

		// Create our popup menus
		cardEditItem.addActionListener (this);
		cardRenameItem.addActionListener (this);
		cardDeleteItem.addActionListener (this);
		cardMoveUpItem.addActionListener (this);
		cardMoveDownItem.addActionListener (this);
		cardPopup.add (cardEditItem);
		cardPopup.add (cardRenameItem);
		cardPopup.add (cardDeleteItem);
		cardPopup.add (new JSeparator());
		cardPopup.add (cardMoveUpItem);
		cardPopup.add (cardMoveDownItem);

		collectionRenameItem.addActionListener (this);
		collectionDeleteItem.addActionListener (this);
		collectionSortItem.addActionListener (this);
		collectionMoveUpItem.addActionListener (this);
		collectionMoveDownItem.addActionListener (this);
		collectionPopup.add (collectionRenameItem);
		collectionPopup.add (collectionDeleteItem);
		collectionPopup.add (new JSeparator());
		collectionPopup.add (collectionSortItem);
		collectionPopup.add (collectionMoveUpItem);
		collectionPopup.add (collectionMoveDownItem);

		indexSortItem.addActionListener (this);
		indexPopup.add (indexSortItem);
		
		// Top panel
		topPanel.setBorder (BorderFactory.createEmptyBorder (5, 5, 5, 5));
		topPanel.setLayout (new BorderLayout());
		topPanel.add (indexPane, BorderLayout.CENTER);

		/*
		 * Set up the title panel, selecting a card will fill this soon.
		 */	
		upButton.setMargin (new Insets (0, 0, 0, 0));
		addAboveButton.setMargin (new Insets (0, 0, 0, 0));
		deleteButton.setMargin (new Insets (0, 0, 0, 0));
		addBelowButton.setMargin (new Insets (0, 0, 0, 0));
		downButton.setMargin (new Insets (0, 0, 0, 0));
		buttonPanel.add (upButton);
		buttonPanel.add (addAboveButton);
		buttonPanel.add (deleteButton);
		buttonPanel.add (addBelowButton);
		buttonPanel.add (downButton);
		buttonPanel.add (doneButton);

		/*
		 * Add listeners.
		 */
		doneButton.addActionListener (this);
		upButton.addActionListener (this);
		addAboveButton.addActionListener (this);
		deleteButton.addActionListener (this);
		addBelowButton.addActionListener (this);
		downButton.addActionListener (this);
		indexTree.addMouseListener (this);
		indexTree.addTreeSelectionListener (this);
		indexTree.getModel().addTreeModelListener (this);

		/*
		 * Enable/disable and configure.
		 */
		DefaultMutableTreeNode rootNode = (DefaultMutableTreeNode) indexModel.getRoot();				
		rootNode.setUserObject (index);

       	for (Enumeration e = index.elements() ; e.hasMoreElements() ;) {
   	   		Collection collection = (Collection) e.nextElement();
   	   		MutableCollectionTreeNode collectionNode = new MutableCollectionTreeNode (collection, true);
   	   		rootNode.add (collectionNode);

	       	for (Enumeration e2 = collection.elements() ; e2.hasMoreElements() ;) {
   	   			Card card = (Card) e2.nextElement();
	   	   		MutableCardTreeNode cardNode = new MutableCardTreeNode (card, false);
   		   		collectionNode.add (cardNode);
	       	}
       	}		
		
       	indexTree.setEditable (true);
		indexTree.setInvokesStopCellEditing (true);
		indexTree.setRootVisible (true);
		indexTree.expandPath (new TreePath (rootNode));
		
		upButton.setEnabled (false);
		addAboveButton.setEnabled (false);
		deleteButton.setEnabled (false);
		addBelowButton.setEnabled (false);
		downButton.setEnabled (false);

		/*
		 * Resize and position ourselves.
		 */
		pack();
		
		Dimension pref = this.getPreferredSize();
		setSize(pref.width * 2, pref.height);	
		
	   	Dimension meSize = getSize();
	   	Dimension momSize = frame.getSize();
	   	Point momPos = frame.getLocationOnScreen();
		int x = momPos.x + (momSize.width / 2) - (meSize.width / 2);
		int y = momPos.y + (momSize.height / 2) - (meSize.height / 2);
   	         
	   	this.setLocation ((x > 0) ? x : 0, (y > 0) ? y : 0);		
	}

	/*
 	 **************************************************************************
	 * Utilities, for use by our listeners.
	 */
	protected void editAtPath (TreePath path) {
		if (path == null) return;		
		DefaultMutableTreeNode node = (DefaultMutableTreeNode) path.getLastPathComponent();
		Vector nodeVector = (Vector) node.getUserObject();
		if (nodeVector instanceof Card) {
			CardDialog cardDialog = new CardDialog (frame, index, (Card) nodeVector);
			cardDialog.setVisible (true);
		} 
	}

	protected void deleteAtPath (TreePath path) {
		if (path == null) return;		
		TreePath parentPath = path.getParentPath();	
		DefaultMutableTreeNode child = (DefaultMutableTreeNode) path.getLastPathComponent();
		DefaultMutableTreeNode parent = (DefaultMutableTreeNode) child.getParent();
		if (parent == null) return;		
		Vector parentVector = (Vector) parent.getUserObject();
		
        int pos = parent.getIndex (child);
           
        indexModel.removeNodeFromParent (child);
        parentVector.remove (pos);
          	
		indexModel.nodeStructureChanged (parent);
			
		indexTree.clearSelection();
	}

	protected void moveAtPath (TreePath path, boolean up) {
		if (path == null) return;		
		TreePath parentPath = path.getParentPath();	
		DefaultMutableTreeNode child = (DefaultMutableTreeNode) path.getLastPathComponent();
		DefaultMutableTreeNode parent = (DefaultMutableTreeNode) child.getParent();
		if (parent == null) return;
		Vector parentVector = (Vector) parent.getUserObject();
		
       	int pos = parent.getIndex (child);
       	if (up && (pos <= 0)) return;
       	if (!up && (pos >= (parent.getChildCount()-1))) return;
           
       	indexModel.removeNodeFromParent (child);
       	parentVector.remove (pos);
          	
       	if (up) {
          	indexModel.insertNodeInto (child, parent, pos-1);
          	parentVector.add (pos-1, child.getUserObject());

        } else {
          	indexModel.insertNodeInto (child, parent, pos+1);
          	parentVector.add (pos+1, child.getUserObject());
        }
		indexModel.nodeStructureChanged (parent);
			
   		indexTree.setSelectionPath (parentPath.pathByAddingChild(child));          		
	}
	
	protected void sortAtPath (TreePath path) {
		if (path == null) return;		
		DefaultMutableTreeNode node = (DefaultMutableTreeNode) path.getLastPathComponent();
		Vector nodeVector = (Vector) node.getUserObject();
		
		if (nodeVector instanceof Collection) {			
			Collection collection = (Collection) nodeVector;			
			for (int i=node.getChildCount()-1; i>=0; i--) {			
				indexModel.removeNodeFromParent ((MutableTreeNode) indexModel.getChild(node, i));
			}

			collection.sortCardsByName ();			
	       	for (Enumeration e2 = collection.elements() ; e2.hasMoreElements() ;) {
   	   			Card card = (Card) e2.nextElement();
	   	   		MutableCardTreeNode cardNode = new MutableCardTreeNode (card, false);
   		   		node.add (cardNode);
	       	}
			indexModel.nodeStructureChanged (node);			

		} else if (nodeVector instanceof Index) {			
			Index index = (Index) nodeVector;			
			for (int i=node.getChildCount()-1; i>=0; i--) {			
				indexModel.removeNodeFromParent ((MutableTreeNode) indexModel.getChild(node, i));
			}

			index.sortCollectionsByName ();			
	       	for (Enumeration e = index.elements() ; e.hasMoreElements() ;) {
   		   		Collection collection = (Collection) e.nextElement();
   	   			MutableCollectionTreeNode collectionNode = new MutableCollectionTreeNode (collection, true);
   	   			node.add (collectionNode);

	       		for (Enumeration e2 = collection.elements() ; e2.hasMoreElements() ;) {
   	   				Card card = (Card) e2.nextElement();
	   	   			MutableCardTreeNode cardNode = new MutableCardTreeNode (card, false);
   		   			collectionNode.add (cardNode);
	       		}
       		}					
			indexModel.nodeStructureChanged (node);	
		}
	}
	
	/*
  	 **************************************************************************
  	 * LISTENERS BELOW HERE
  	 **************************************************************************
  	 */
	/*
  	 **************************************************************************
	 * Tree selection listeners.
	 */
	public void valueChanged(TreeSelectionEvent e) {
		TreePath newPath = e.getNewLeadSelectionPath();
		
		if (newPath == null) {
			upButton.setEnabled (false);
			addAboveButton.setEnabled (false);
			deleteButton.setEnabled (false);
			addBelowButton.setEnabled (false);
			downButton.setEnabled (false);

		} else {
			DefaultMutableTreeNode node = (DefaultMutableTreeNode) newPath.getLastPathComponent();
			Object userData = node.getUserObject();
		
			if (userData instanceof Index) {
				upButton.setEnabled (false);
				addAboveButton.setEnabled (true);
				deleteButton.setEnabled (false);
				addBelowButton.setEnabled (true);
				downButton.setEnabled (false);

			} else if (userData instanceof Collection) {
				TreeNode parent = node.getParent();
				
				upButton.setEnabled (parent.getIndex(node) > 0);
				addAboveButton.setEnabled (true);
				deleteButton.setEnabled (true);
				addBelowButton.setEnabled (true);
				downButton.setEnabled (parent.getIndex(node) < (parent.getChildCount() - 1));

			} else if (userData instanceof Card) {
				TreeNode parent = node.getParent();

				upButton.setEnabled (parent.getIndex(node) > 0);
				addAboveButton.setEnabled (false);
				deleteButton.setEnabled (true);
				addBelowButton.setEnabled (false);
				downButton.setEnabled (parent.getIndex(node) < (parent.getChildCount() - 1));
			}
		}
	}

	/*
 	 **************************************************************************
	 * Act on our button actions.
	 */
	public void actionPerformed(ActionEvent e) {
		Object source = e.getSource();

		// Done enough damage, close the screen.
		if (source == doneButton) {
			this.setVisible (false);

		// Move selected item up
		} else if ((source == upButton) ||
				   (source == cardMoveUpItem) ||
				   (source == collectionMoveUpItem)) {		
			moveAtPath (indexTree.getSelectionPath(), true);
			
		// Move selected item up
		} else if ((source == downButton) ||
				   (source == cardMoveDownItem) ||
				   (source == collectionMoveDownItem)) {		
			moveAtPath (indexTree.getSelectionPath(), false);
			
		// Delete selected item
		} else if ((source == deleteButton) ||
				   (source == cardDeleteItem) ||
				   (source == collectionDeleteItem)) {		

			deleteAtPath (indexTree.getSelectionPath());

		// Create a new collection or card as the first or last of this node's children
		} else if ((source == addAboveButton) || (source == addBelowButton)) {
			TreePath path = indexTree.getSelectionPath();
			if (path == null) {
				return;
			}
			
			DefaultMutableTreeNode node = (DefaultMutableTreeNode) path.getLastPathComponent();
			Vector nodeVector = (Vector) node.getUserObject();
			DefaultMutableTreeNode newNode = null;

			if (nodeVector instanceof Index) {			
				Collection collection = new Collection (index, "New Collection");
				newNode = new MutableCollectionTreeNode (collection, true);
				
			} else {
				Card card = new Card ("New Card");
				newNode = new MutableCardTreeNode (card, false);
			}
			
			if (source == addAboveButton) {
          		indexModel.insertNodeInto (newNode, node, 0);
          		nodeVector.add (0, newNode.getUserObject());

			} else {
          		indexModel.insertNodeInto (newNode, node, indexModel.getChildCount (node));
          		nodeVector.add (nodeVector.size(), newNode.getUserObject());
			}

			indexModel.nodeStructureChanged (node);

			TreePath newPath = path.pathByAddingChild(newNode);
          	indexTree.setSelectionPath (newPath);
          	indexTree.startEditingAtPath (newPath);
        
        // Both rename popup buttons do the same thing.
		} else if ((source == cardRenameItem) || (source == collectionRenameItem)) {
			TreePath path = indexTree.getSelectionPath();
			if (path != null) {
				indexTree.startEditingAtPath (path);
			}

		// Edit, with a dialog, not just rename
		} else if (source == cardEditItem) {			
			editAtPath (indexTree.getSelectionPath());

		// Edit, with a dialog, not just rename
		} else if ((source == indexSortItem) ||
				   (source == collectionSortItem)) {			
				   	
			sortAtPath (indexTree.getSelectionPath());
		}
	}

	/*
 	 **************************************************************************
	 * Act on tree node changes.
	 */
 	public void treeNodesChanged(TreeModelEvent e) {
 		Object children[] = e.getChildren();
 		
 		for (int i=0; i<children.length; i++) { 			
 			DefaultMutableTreeNode node = (DefaultMutableTreeNode) children[i];
			Object userData = node.getUserObject();
			
			if (userData instanceof Collection) {
				Collection collection = (Collection) userData;
				collection.setName (node.toString());

			} else if (userData instanceof Card) {
				Card card = (Card) userData;
				card.setName (node.toString());

			} else {
				System.out.println (userData.getClass());
			}
 		}
	}
 	public void treeNodesInserted(TreeModelEvent e) {}
 	public void treeNodesRemoved(TreeModelEvent e) {	}
 	public void treeStructureChanged(TreeModelEvent e) {}
 	 	 
	/*
  	 **************************************************************************
	 * Mouse listeners.
	 */
 	public void mouseClicked(MouseEvent e) {
 		Object source = e.getSource();
 		if (source == indexTree) {
			TreePath path = indexTree.getPathForLocation (e.getPoint().x, e.getPoint().y);
			if (path == null) {
				return;
			}

			/*
			 * Double-click, invoke a dialog if we have one.
			 */
 			if (((e.getModifiers() & InputEvent.BUTTON1_MASK) != 0) && 
 				e.getClickCount() >= 2) {
 					
 				editAtPath (path);

 			/*
 			 * Set the right-clicked item to be selected, I hate it when interfaces
			 * don't do this.  It's confusing, because you never know if you're action
			 * will affect the selected or the right click target object.  Then popup.
 			 */
 			} else if (((e.getModifiers() & InputEvent.BUTTON3_MASK) != 0) && 
 					   e.getClickCount() == 1) {
 					   	
 				indexTree.setSelectionPath (path);

				DefaultMutableTreeNode node = (DefaultMutableTreeNode) path.getLastPathComponent();
				Vector nodeVector = (Vector) node.getUserObject();
 				if (nodeVector instanceof Card) {
	 				cardPopup.show (indexTree, e.getPoint().x, e.getPoint().y);

 				} else if (nodeVector instanceof Collection) {
	 				collectionPopup.show (indexTree, e.getPoint().x, e.getPoint().y);

 				} else if (nodeVector instanceof Index) {
	 				indexPopup.show (indexTree, e.getPoint().x, e.getPoint().y);
 				}
 			}
 		}
 	}
  	public void mouseEntered(MouseEvent e) {}
  	public void mouseExited(MouseEvent e) {}
  	public void mousePressed(MouseEvent e) {}
  	public void mouseReleased(MouseEvent e) {}
}