/******************************************************************************
 * Module Name : $RCSfile: MainFrame.java,v $
 *
 * Description : Main frame for the stand-up comic collection program.
 *
 * Updates:
 *   $Log: MainFrame.java,v $
 *   Revision 1.32  2002/07/13 01:06:10  jcouper
 *   job:157
 *   Add +/- for grades.  (Also fixed 147 at the same time).
 *
 *   Revision 1.31  2002/07/07 00:56:38  jcouper
 *   Abandoned for now the idea of a "view all" card.  Removed legacy code.
 *
 *   Revision 1.30  2001/10/11 21:28:04  jcouper
 *   142: Added _ALT option to Issue menu
 *
 *   Revision 1.29  2001/10/07 05:50:02  jcouper
 *   138: Get new card info when card is deleted
 *
 *   Revision 1.28  2001/07/29 00:47:27  jcouper
 *   job:137
 *   Added support for credits, and re-vamped the EditDialog.
 *
 *   Revision 1.27  2001/07/11 08:36:41  jcouper
 *   Scroll down when appending a new issue
 *
 *   Revision 1.26  2001/07/11 08:20:26  jcouper
 *   Scroll card table when up/down issues
 *
 *   Revision 1.25  2001/07/11 07:22:31  jcouper
 *   Add confirmation dialog for delete, and filter for titles
 *
 *   Revision 1.24  2001/07/09 01:45:11  jcouper
 *   Update version to 1.0.4, remove broken help links
 *
 *   Revision 1.23  2001/06/17 06:52:19  jcouper
 *   job:118,119
 *   Not cleaning up previous collection menu properly.
 *
 *   Revision 1.22  2001/06/16 01:49:08  jcouper
 *   Commit of final release 1.0.3
 *
 *   Revision 1.21  2001/04/20 00:54:50  jcouper
 *   Release 1.0.3
 *
 *   Revision 1.20  2001/04/11 08:22:24  jcouper
 *   Add Filter Support
 *
 *   Revision 1.19  2001/04/04 07:34:50  jcouper
 *   Support credits, filters, new columns, etc, etc.  Stable release.
 *
 *   Revision 1.18  2000/08/23 19:34:54  jcouper
 *   Allow editing of card on double click.
 *
 *   Revision 1.17  2000/03/19 04:41:26  Jonathan
 *   Repaint cards after editing
 *
 *   Revision 1.16  2000/03/18 00:46:04  Jonathan
 *   Big changes in the way the IndexDialog works, including a new CardDialog.
 *   New preferences for column display.  Permit renaming of title files.
 *
 *   Revision 1.15  2000/02/19 00:18:48  Jonathan
 *   Add saving of settings.
 *
 *   Revision 1.14  2000/01/10 02:22:20  Jonathan
 *   v1.0.0
 *
 *   Revision 1.13  2000/01/09 00:11:55  Jonathan
 *   Edit Dialog, Preferences
 *
 *   Revision 1.12  2000/01/07 05:43:34  Jonathan
 *   Render fixed cell dates differently
 *
 *   Revision 1.11  1999/11/27 10:31:29  Jonathan
 *   Interface tweaks.  Reporting on the menu.  Fooling around with HTML
 *   about box.
 *
 *   Revision 1.10  1999/11/15 22:26:52  Jonathan
 *   Report stuff compiles at least.  Doesn't run, mind you.
 *
 *   Revision 1.9  1999/10/25 08:33:07  Jonathan
 *   v0_9_2
 *
 *   Revision 1.8  1999/10/24 22:37:54  Jonathan
 *   Proper CVS headers.  Fix A-018/19/20
 *
 *
 * Copyright Notice:
 * (c) 1999. This source code was developed by Jonathan Couper and friends.
 *            You may use it for any purpose.  No guarantees given or implied.
 *****************************************************************************/
package standup;

import java.util.Vector;
import java.util.Enumeration;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.event.*;
import javax.swing.*;
import javax.swing.table.*;
import javax.swing.text.*;
import javax.swing.filechooser.FileFilter;
import standup.parse.*;
import standup.views.*;
import standup.util.*;
import standup.report.*;

public class MainFrame extends JFrame
    implements ChangeListener, ActionListener, WindowListener,
        MouseListener
{
    // Application Settings
    ApplicationSettings applicationSettings = Application.getSettings ();

    JTabbedPane cardTabPane = new JTabbedPane();

    EditDialog editDialog = new EditDialog(this);
    JMenuBar menuBar = new JMenuBar();

    JMenu fileMenu = new JMenu("File");
    JMenuItem fileOpenItem = new JMenuItem("Open...");
    JMenuItem filePreferencesItem = new JMenuItem("Preferences...");
    JMenuItem fileReportItem = new JMenuItem("Report...");
    JMenuItem fileSaveItem = new JMenuItem("Save");
    JMenuItem fileExitItem = new JMenuItem("Exit");

    JMenu collectionMenu = new JMenu("Collections");
    JMenuItem collectionIndexItem = new JMenuItem ("Index...");
    ButtonGroup collectionButtonGroup = new ButtonGroup();

    JMenu cardMenu = new JMenu("Cards");
    JMenuItem cardSortItem = new JMenuItem ("Sort A..Z");
    JMenuItem cardNewItem = new JMenuItem ("New...");
    JMenuItem cardEditItem = new JMenuItem ("Edit...");
    JMenuItem cardDeleteItem = new JMenuItem ("Delete");

    JMenu issueMenu = new JMenu("Issues");
    JMenuItem issueTitleItem = new JMenuItem ("Title...");
    JMenuItem issueEditItem = new JMenuItem ("Edit...");
    JMenu issueNewMenu = new JMenu ("New");
    JMenuItem issueAppendItem = new JMenuItem ("<Append>");
    JMenuItem issueAlternateItem = new JMenuItem ("<Alternate>");
    JMenu issueGradeMenu = new JMenu ("Grade");

    JMenu helpMenu = new JMenu("Help");
    JMenuItem helpAboutItem = new JMenuItem("About");

    File indexFile = null;
    Index index = null;
    Collection currentCollection = null;
    Card currentCard = null;

    /*
     * Edited title ref and issue.
     */
    FullCardTableModel selectedTableModel = null;
    int editedRow = -1;
    TitleRef editedTitleRef = null;
    Issue editedIssue = null;

    /*
     * Creates the main frame, with Collection/Card selection panel up the top,
     * card display panel in center, and editing panel at the bottom.
     */
    public MainFrame () {
        super (Application.getAppName());

        cardTabPane.setEnabled(false);
        cardTabPane.addChangeListener(this);

        getContentPane().setLayout (new BorderLayout());
        getContentPane().add (cardTabPane, BorderLayout.CENTER);

        /*
         * Misc Listeners.
         */
        cardTabPane.addMouseListener (this);

        /*
         * Construct our file menu.
         */
        fileOpenItem.addActionListener(this);
        fileOpenItem.setAccelerator(KeyStroke.getKeyStroke
            (java.awt.event.KeyEvent.VK_O, java.awt.Event.CTRL_MASK));

        fileSaveItem.addActionListener(this);
        fileSaveItem.setAccelerator(KeyStroke.getKeyStroke
            (java.awt.event.KeyEvent.VK_S, java.awt.Event.CTRL_MASK));

        filePreferencesItem.addActionListener(this);
        filePreferencesItem.setAccelerator(KeyStroke.getKeyStroke
            (java.awt.event.KeyEvent.VK_P, java.awt.Event.CTRL_MASK));

        fileReportItem.addActionListener(this);
        fileReportItem.setAccelerator(KeyStroke.getKeyStroke
            (java.awt.event.KeyEvent.VK_R, java.awt.Event.CTRL_MASK));

        fileExitItem.addActionListener(this);
        fileMenu.add(fileOpenItem);
        fileMenu.add(fileSaveItem);
        fileMenu.add(new JSeparator());
        fileMenu.add(filePreferencesItem);
        fileMenu.add(fileReportItem);
        fileMenu.add(new JSeparator());
        fileMenu.add(fileExitItem);

        /*
         * Preconstruct a little of our collection menu.
         * The rest is done on the fly.
         */
        collectionIndexItem.setEnabled(false);
        collectionIndexItem.addActionListener(this);
        collectionIndexItem.setAccelerator (KeyStroke.getKeyStroke
            (java.awt.event.KeyEvent.VK_I, java.awt.Event.CTRL_MASK));

        collectionMenu.add(collectionIndexItem);
        collectionMenu.add(new JSeparator());

        /*
         * Construct our card menu.
         */
        cardSortItem.setEnabled(false);
        cardSortItem.addActionListener(this);
        cardSortItem.setAccelerator (KeyStroke.getKeyStroke (java.awt.event.KeyEvent.VK_Z, java.awt.Event.CTRL_MASK));

        cardNewItem.setEnabled(false);
        cardNewItem.addActionListener(this);
        cardNewItem.setAccelerator (KeyStroke.getKeyStroke (java.awt.event.KeyEvent.VK_N, java.awt.Event.CTRL_MASK));

        cardEditItem.setEnabled(false);
        cardEditItem.addActionListener(this);
        cardEditItem.setAccelerator (KeyStroke.getKeyStroke (java.awt.event.KeyEvent.VK_E, java.awt.Event.CTRL_MASK));

        cardDeleteItem.setEnabled(false);
        cardDeleteItem.addActionListener(this);
        cardDeleteItem.setAccelerator (KeyStroke.getKeyStroke (java.awt.event.KeyEvent.VK_D, java.awt.Event.CTRL_MASK));

        cardMenu.add(cardSortItem);
        cardMenu.add(new JSeparator());
        cardMenu.add(cardNewItem);
        cardMenu.add(cardEditItem);
        cardMenu.add(cardDeleteItem);

        /*
         * Construct our issue menu.
         */
        issueTitleItem.setEnabled(false);
        issueTitleItem.addActionListener(this);
        issueTitleItem.setAccelerator (KeyStroke.getKeyStroke (java.awt.event.KeyEvent.VK_T, java.awt.Event.CTRL_MASK));

        issueEditItem.setEnabled(false);
        issueEditItem.addActionListener(this);
        issueEditItem.setAccelerator (KeyStroke.getKeyStroke (java.awt.event.KeyEvent.VK_ENTER, java.awt.Event.CTRL_MASK));

        issueAppendItem.setEnabled(false);
        issueAppendItem.addActionListener(this);

        issueAlternateItem.setEnabled(false);
        issueAlternateItem.addActionListener(this);

        // Grade Menu
        issueGradeMenu.add(gradeMenuItem (java.awt.event.KeyEvent.VK_0, Issue.GRADE_NONE));
        issueGradeMenu.add(gradeMenuItem (java.awt.event.KeyEvent.VK_1, Issue.GRADE_NEAR_MINT));
        issueGradeMenu.add(gradeMenuItem (java.awt.event.KeyEvent.VK_2, Issue.GRADE_VERY_FINE));
        issueGradeMenu.add(gradeMenuItem (java.awt.event.KeyEvent.VK_3, Issue.GRADE_FINE));
        issueGradeMenu.add(gradeMenuItem (java.awt.event.KeyEvent.VK_4, Issue.GRADE_VERY_GOOD));
        issueGradeMenu.add(gradeMenuItem (java.awt.event.KeyEvent.VK_5, Issue.GRADE_GOOD));
        issueGradeMenu.add(gradeMenuItem (java.awt.event.KeyEvent.VK_6, Issue.GRADE_FAIR));
        issueGradeMenu.add(gradeMenuItem (java.awt.event.KeyEvent.VK_7, Issue.GRADE_POOR));
        issueGradeMenu.add(gradeMenuItem (java.awt.event.KeyEvent.VK_8, Issue.GRADE_REPRINT));
        issueGradeMenu.add(gradeMenuItem (java.awt.event.KeyEvent.VK_9, Issue.GRADE_ON_ORDER));

        // Issue Menu
        issueMenu.add(issueTitleItem);
        issueMenu.add(new JSeparator());
        issueMenu.add(issueEditItem);
        issueMenu.add(issueNewMenu);
        issueMenu.add(issueGradeMenu);

        // Issue->New Menu
        issueNewMenu.add(issueAppendItem);
        issueNewMenu.add(issueAlternateItem);

        // Collection Menu
        helpAboutItem.setEnabled (true);
        helpAboutItem.addActionListener(this);
        helpMenu.add(helpAboutItem);

        // Main Menu
        menuBar.add(fileMenu);
        menuBar.add(collectionMenu);
        menuBar.add(cardMenu);
        menuBar.add(issueMenu);
        menuBar.add(helpMenu);
        setJMenuBar(menuBar);

        /*
         * Nothing is selected.  This de-activates buttons.
         */
        noIssueSelected ();

        /*
         * Define our size, and get everything to fit.
         */
        setLocation(20,20);
        setSize(700,500);
        addWindowListener(this);
        getContentPane().validate();

        setFrameName ();
    }

    /*
     **************************************************************************
     * CONFIRMATION DIALOG
     **************************************************************************
     */
    protected boolean getConfirm (String message, String title) {
        int result = JOptionPane.showConfirmDialog (
            this, message, title,
            JOptionPane.YES_NO_OPTION,
            JOptionPane.QUESTION_MESSAGE,
            new ImageIcon (getClass().getResource ("images/about.gif")));

        return (result == JOptionPane.OK_OPTION);
    }

    /*
     **************************************************************************
     * QUICK MENU CREATOR
     **************************************************************************
     */
    protected JMenuItem gradeMenuItem (int vk, int grade) {
        String code = Issue.getAbbreviationForGrade (grade);
        JMenuItem menuItem = new JMenuItem (code);
        menuItem.addActionListener (editDialog);
        menuItem.setAccelerator (KeyStroke.getKeyStroke (vk, java.awt.Event.CTRL_MASK));

        return menuItem;
    }

    /*
     **************************************************************************
     * INDEX MANAGEMENT METHODS.
     **************************************************************************
     * Select an index file, or return null.
     */
    public File selectIndexFile () {
        JFileChooser indexChooser = new JFileChooser("data");
        indexChooser.setFileFilter (new IndexFilter());
        indexChooser.setFileSelectionMode (JFileChooser.FILES_ONLY);
        indexChooser.setMultiSelectionEnabled (false);

        if (indexChooser.showOpenDialog(this) != JFileChooser.APPROVE_OPTION) {
            return null;
        }

        File newIndexFile = indexChooser.getSelectedFile();

        return newIndexFile;
    }

    private class IndexFilter extends FileFilter {
        public boolean accept (File f) {
            return (f.getName().endsWith(".idx") || f.isDirectory());
        }

        public String getDescription() {
            return "Comic Index Files";
        }
    }

    /*
     * Clear collection, collection list, tables, etc.  Scrubs everything,
     * except for the index edit button.
     */
    public void removeCollectionComponents() {
        currentCollection = null;
        collectionMenu.setEnabled (false);
        collectionIndexItem.setEnabled (false);
        cardNewItem.setEnabled (false);
        cardSortItem.setEnabled (false);

        noIssueSelected ();

        /* Remove all except the Index item and Separator */
        for (Enumeration e = collectionButtonGroup.getElements() ; e.hasMoreElements() ;) {
            collectionButtonGroup.remove ((JRadioButtonMenuItem) e.nextElement());
        }
        for (int i=collectionMenu.getItemCount() - 1; i >= 2; i--) {
            collectionMenu.remove (i);
        }
        if (cardTabPane.getTabCount() > 0) {
            cardTabPane.removeAll();
        }
    }


    /*
     * Create collection, collection list, tables, etc.
     */
    public void createCollectionComponents (Collection collectionToSelect) {
        for (Enumeration e = index.elements() ; e.hasMoreElements() ;) {
            Collection collection = (Collection) e.nextElement();

            JRadioButtonMenuItem menuItem = new JRadioButtonMenuItem (collection.toString());
            if (currentCollection == null) {
                if ((collectionToSelect == null) || (collectionToSelect == collection)) {
                    menuItem.setSelected (true);
                    setCollection (collection);
                }
            }

            menuItem.addActionListener (new CollectionListener (this, collection));
            collectionMenu.add (menuItem);
            collectionButtonGroup.add (menuItem);
        }

        collectionMenu.setEnabled (index.size() > 0);
        collectionIndexItem.setEnabled (true);
    }

    /*
     * Fill the collection menu, based on the current index file.
     */
    public void fillIndex (boolean tryDefault) {
        /*
         * Keep the old index file, to revert to if the user cancels.
         */
        File previousIndexFile = indexFile;
        indexFile = null;

        /*
         * Looking for an index file.
         * - Try last used (if default is specified)
         * - Try current dir (if default is specified)
         * - Try asking
         * - Give up, go back to the last one.
         */
        if (tryDefault && (indexFile == null)) {
            File lastIndexFile = new File (applicationSettings.lastIndex);
            if (lastIndexFile.canRead()) {
                indexFile = lastIndexFile;
            }
        }
        if (tryDefault && (indexFile == null)) {
            File defaultIndexFile = new File ("data/default.idx");
            if (defaultIndexFile.canRead()) {
                indexFile = defaultIndexFile;
            }
        }
        if (indexFile == null) {
            indexFile = selectIndexFile();
        }
        if (indexFile == null) {
            indexFile = previousIndexFile;
            return;
        }

        /*
         * See if we need to save the old one first.
         */
        save ();

        /*
         * Get rid of the old collection and index components.
         */
        removeCollectionComponents ();

        /*
         * Load the new index, and configure it.
         */
        index = new Index (indexFile);

        /*
         * Create components for the new collections.
         */
        createCollectionComponents (null);

        /*
         * Set the frame name.
         */
        setFrameName ();
    }

    /*
     **************************************************************************
     * COLLECTION, CARD, and ISSUE MANAGEMENT METHODS.
     **************************************************************************
     * Set a new collection, then update ourself.
     */
    public void setCollection (Collection collection) {
        currentCollection = collection;
        updateCollectionDisplay ();
        cardNewItem.setEnabled (collection != null);
        cardSortItem.setEnabled (collection != null);
    }

    public void setFrameName () {
        if (index == null) {
            this.setTitle (Application.getAppName());

        } else if (currentCollection == null) {
            this.setTitle (Application.getAppName() + " - " +
                            indexFile.getName());

        } else {
            this.setTitle (Application.getAppName() + " - " +
                            indexFile.getName() + ", " +
                            currentCollection.getName());
        }
    }

    public void addTabForCard (Card card) {
        /*
         * Create the card table and associated model.  Set the column widths.
         */
        FullCardTableModel tableModel = new FullCardTableModel(card);

        JTable cardTable = new JTable(tableModel);
        cardTable.clearSelection();
        cardTable.getColumnModel().getColumn(0).setCellRenderer (new DateCellRenderer());

        JScrollPane scrollPane = new JScrollPane (cardTable);

        /*
         * Set the column widths.
         */
        tableModel.setPreferredWidths (cardTable);

        /*
         * Do some stuff with the list selection mode.  Single select, listened for.
         */
        FullCardListener listener = new FullCardListener (cardTable, this);

        ListSelectionModel listSelectionModel = cardTable.getSelectionModel();
        listSelectionModel.setSelectionMode (ListSelectionModel.SINGLE_SELECTION);
        listSelectionModel.addListSelectionListener(listener);

        cardTable.addMouseListener (listener);

        cardTabPane.addTab(card.getName(), scrollPane);
    }

    /*
     * Are we out of date?  Reload all our cards.  It's a bit of work, but it's
     * the only way.
     */
    public void updateCollectionDisplay () {
        if (cardTabPane.getTabCount() > 0) {
            cardTabPane.removeAll();
        }

        /*
         * We create a JTable, JScrollPane and FullCardTableModel for each tab.
         * I tried re-using them, but it just lead to tears.  This is the proper
         * way.
         */
        if (currentCollection != null) {
            for (Enumeration en2 = currentCollection.elements() ; en2.hasMoreElements() ;) {
                addTabForCard ((Card) en2.nextElement());
            }
            cardTabPane.setEnabled(true);

        } else {
            cardTabPane.setEnabled(false);
        }

        /*
         * Rename the frame, just in case the collection name changed.
         */
        setFrameName ();
        editDialog.setEmpty ();
    }

    /*
     * Issue Selected.  Edit the indicated row in the table model.
     */
    public void issueSelected (JTable cardTable) {
        if (selectedTableModel == null) {
            System.out.println ("issueSelected: No table");
            return;
        }
        int row = cardTable.getSelectedRow ();

        if (row < 0) {
            noIssueSelected ();

        } else {
            if (selectedTableModel != (FullCardTableModel) cardTable.getModel()) {
                System.out.println ("issueSelected: Table model mismatch");
                System.exit (1);
            }
            editedRow = row;

            editedTitleRef = selectedTableModel.getCard().titleRefAtArrayPos(row);
            editedIssue = selectedTableModel.getCard().issueAtArrayPos(row);

            issueEditItem.setEnabled (true);
            issueGradeMenu.setEnabled (true);
            issueTitleItem.setEnabled (true);

            issueAppendItem.setText (String.valueOf (editedTitleRef.getLastIssue().getCodeAsInt() + 1));
            issueAppendItem.setEnabled (!editedTitleRef.isFiltered() &&
                                          (editedTitleRef.getLastIssue() != null) &&
                                          editedTitleRef.getLastIssue().isCodeNumeric());

            issueAlternateItem.setText (editedIssue.getCode() + "_ALT");
            issueAlternateItem.setEnabled (!editedTitleRef.isFiltered() &&
                                           editedIssue.isCodeNumeric());

            issueNewMenu.setEnabled (issueAppendItem.isEnabled() ||
                                     issueAlternateItem.isEnabled());

            editDialog.editNow (editedTitleRef, editedIssue);
        }
    }

    public void noIssueSelected () {
        editedRow = -1;
        editedTitleRef = null;
        editedIssue = null;

        editDialog.setEmpty ();
        issueEditItem.setEnabled (false);
        issueNewMenu.setEnabled (false);
        issueGradeMenu.setEnabled (false);
        issueTitleItem.setEnabled (false);
        issueAppendItem.setEnabled (false);
    }

    public void showEditDialog () {
        if (!editDialog.isVisible()) {
            editDialog.setModal (false);
            editDialog.setVisible (true);

        } else {
            editDialog.toFront ();
        }
    }

    public void setEditedCard() {
        int pos = cardTabPane.getSelectedIndex();

        if (pos < 0) {
            currentCard = null;
            cardEditItem.setEnabled (false);
            cardDeleteItem.setEnabled (false);

            noIssueSelected ();

        } else {
            JScrollPane scrollPane = (JScrollPane) cardTabPane.getComponentAt (pos);
            JTable cardTable = (JTable) scrollPane.getViewport().getView();
            selectedTableModel = (FullCardTableModel) cardTable.getModel();
            Card selectedCard = (Card) currentCollection.elementAt (pos);

            currentCard = selectedCard;
            cardEditItem.setEnabled (true);
            cardDeleteItem.setEnabled (true);

            issueSelected (cardTable);
        }
    }

    /*
     * Edit a card.  Because of the delay in calling up the card,
     * we can have problems where we try to invoke multiple cards
     * at once.  Also, we can end up selecting a card in the background,
     * with the result that the wrong tab is renamed upon return.
     *
     * The editingCardIndex variable deals with all these problems.
     */
    protected static int editingCardIndex = -1;
    protected static Card editingCard = null;
    public void showCardDialog () {
        if (currentCard == null) {
            return;
        }
        if (editingCardIndex >= 0) {
            return;
        }
        editingCardIndex = cardTabPane.getSelectedIndex();
        editingCard = currentCard;

        setCursor (new Cursor (Cursor.WAIT_CURSOR));
        CardDialog cardDialog = new CardDialog (this, index, editingCard);
        setCursor (new Cursor (Cursor.DEFAULT_CURSOR));

        cardDialog.setVisible (true);
        fireTableDataChanged();

        cardTabPane.setTitleAt (editingCardIndex, editingCard.getName());
        editingCardIndex = -1;
    }

    public void fireUpdateEditedRow () {
        if (selectedTableModel == null) {
            System.out.println ("fireUpdateEditedRow: No table");
            return;
        }
        if (editedRow < 0) {
            System.out.println ("fireUpdateEditedRow: No row");
            return;
        }
        selectedTableModel.fireTableRowsUpdated (editedRow, editedRow);
    }

    public void fireUpdateAllRows () {
        if (selectedTableModel == null) {
            System.out.println ("fireUpdateAllRows: No table");
            return;
        }
        selectedTableModel.fireTableRowsUpdated (0, selectedTableModel.getRowCount() - 1);
    }

    public void fireTableDataChanged () {
        if (selectedTableModel == null) {
            System.out.println ("fireTableDataChanged: No table");
            return;
        }
        selectedTableModel.fireTableDataChanged ();
    }

    /*
     * Selection utilities on the card table.
     */
    public JTable getSelectedTable () {
        JScrollPane scrollPane = (JScrollPane) cardTabPane.getComponentAt (cardTabPane.getSelectedIndex());
        return (JTable) scrollPane.getViewport().getView();
    }

    public int getSelectedRow () {
        JTable cardTable = getSelectedTable ();
        return cardTable.getSelectedRow ();
    }

    public void selectDeltaRow (int row, int delta) {
        if (row < 0) {
            return;
        }
        JScrollPane scrollPane = (JScrollPane) cardTabPane.getComponentAt (cardTabPane.getSelectedIndex());
        JScrollBar verticalBar = scrollPane.getVerticalScrollBar ();
        JTable cardTable = (JTable) scrollPane.getViewport().getView();

        row = row + delta;
        if ((row < 0) || (row >= cardTable.getModel().getRowCount())) {
            return;
        }
        cardTable.addRowSelectionInterval (row, row);
        verticalBar.setValue (verticalBar.getValue () + delta * verticalBar.getUnitIncrement (delta));
    }

    public void selectDeltaRow (int delta) {
        selectDeltaRow (getSelectedRow (), delta);
    }

    /*
     **************************************************************************
     * SAVE AND EXIT METHODS.
     **************************************************************************
     * Ask if the user wants to save any titles.
     */
    public void save () {
        // Save settings first.
        Application.saveSettings();

        // Now our index and titles.
        if (index == null) {
            return;
        }
        Vector allTitles = index.getAllTitles ();
        Vector saveList = new Vector();

        /*
         * Create the list of possible save things to offer to the user.
         * This includes the index, and all of the titles.
         */
        if (index.isModified()) {
            saveList.addElement (index);
        }
        for (Enumeration e = allTitles.elements() ; e.hasMoreElements() ;) {
            Title title = (Title) e.nextElement ();
            if (title.isModified()) {
                saveList.addElement (title);
            }
        }

        if (saveList.size() > 0) {
            CheckSelectDialog selectDialog = new CheckSelectDialog (this, "Save File?", saveList);
            selectDialog.setVisible (true);

            if (selectDialog.didUserPressOK()) {
                for (int i=0; i<saveList.size(); i++) {
                    if (selectDialog.getCheckState (i)) {
                        Object saveThing = saveList.elementAt (i);
                        if (saveThing instanceof Index) {
                            Index saveIndex = (Index) saveThing;
                            saveIndex.save ();

                        } else if (saveThing instanceof Title) {
                            Title saveTitle = (Title) saveThing;
                            saveTitle.save ();
                        }
                    }
                }
            }
        }
    }

    /*
     * Exit routine.  We save anything which needs saving, of course.
     */
    public void doExit () {
        save();
        System.exit(0);
    }

    /*
     **************************************************************************
     * LISTENERS BELOW HERE
     **************************************************************************
     */
    /*
     **************************************************************************
     * Change listeners.  Tabs do this.  Nothing else does.
     */
    public void stateChanged(ChangeEvent e) {
        setEditedCard();
    }

    /*
     **************************************************************************
     * Act on our button actions.
     */
    public void actionPerformed(ActionEvent e) {
        Object source = e.getSource();

        /*
         * FILE Functions.
         */
        if (source == fileExitItem) {
            doExit();

        } else if (source == fileSaveItem) {
            save();

        } else if (source == filePreferencesItem) {
            PreferencesDialog preferencesDialog = new PreferencesDialog (this);
            preferencesDialog.setVisible (true);

        } else if (source == fileReportItem) {
            ReportDialog reportDialog = new ReportDialog (this, index);
            reportDialog.setVisible (true);

        } else if (source == fileOpenItem) {
            fillIndex (false);

        /*
         * COLLECTION Functions.
         */
        } else if (source == collectionIndexItem) {
            IndexDialog indexDialog = new IndexDialog (this, index);
            indexDialog.setVisible (true);

            Collection oldCurrentCollection = currentCollection;

            removeCollectionComponents ();
            createCollectionComponents (oldCurrentCollection);
            updateCollectionDisplay ();

            index.setModified (true);

            setFrameName ();

        } else if (source instanceof Collection) {
            setCollection ((Collection) source);

        /*
         * CARD Functions.
         */
        } else if (source == cardEditItem) {
            if (currentCard == null) {
                return;
            }
            showCardDialog ();

        } else if (source == cardDeleteItem) {
            if (currentCard == null) {
                return;
            }
            if (! getConfirm ("Delete Card '" + currentCard + "'?", "Delete Confirmation")) {
                return;
            }

            // Get rid of that card
            index.setModified (true);
            currentCard.clear ();
            currentCollection.remove (currentCard);
            cardTabPane.removeTabAt (cardTabPane.getSelectedIndex ());

            // Set for editing the new card.  This SHOULD happen automatically
            // via the selectionChanged listener on the tabbed pane, but it doesn't.
            setEditedCard();

        } else if (source == cardSortItem) {
            if (currentCollection == null) {
                return;
            }
            index.setModified (true);
            currentCollection.sortCardsByName ();
            setCollection (currentCollection);

        } else if (source == cardNewItem) {
            if (currentCollection == null) {
                return;
            }
            index.setModified (true);
            Card newCard = new Card ("New Card");
            currentCollection.addElement (newCard);
            addTabForCard (newCard);
            cardTabPane.setSelectedIndex (cardTabPane.getTabCount() - 1);
            showCardDialog ();

        /*
         * ISSUE Functions.
         */
        } else if (source == issueTitleItem) {
            TitleDialog titleDialog = new TitleDialog (this, editedTitleRef);
            titleDialog.setVisible (true);

            editedTitleRef.getTitle().computeDerivedDates();
            fireTableDataChanged();

        } else if (source == issueEditItem) {
            showEditDialog ();

        } else if (source == issueAppendItem) {
            Title title = editedTitleRef.getTitle();
            title.cloneLastIssue ();

            /* Store the current row, we're gonna dork with the table data. */
            int row = getSelectedRow ();

            /* This will clear our selected, but we've saved it! */
            fireTableDataChanged();

            /* We know we added one, so scroll down by a row */
            selectDeltaRow (row, 1);

        } else if (source == issueAlternateItem) {
            Title title = editedTitleRef.getTitle();
            title.makeAlternateIssue (editedIssue);

            /* Store the current row, we're gonna dork with the table data. */
            int row = getSelectedRow ();

            /* This will clear our selected, but we've saved it! */
            fireTableDataChanged();

            /* We know we added one, so scroll down by a row */
            selectDeltaRow (row, 1);

        /*
         * HELP Functions.
         */
        } else if (source == helpAboutItem) {
            AboutDialog aboutDialog = new AboutDialog (this);
            aboutDialog.setVisible (true);
        }
    }

    /*
     **************************************************************************
     * Window listeners
     */
    public void windowActivated(WindowEvent e) {}
    public void windowClosed(WindowEvent e) {}
    public void windowClosing(WindowEvent e) {
        doExit();
    }
    public void windowDeactivated(WindowEvent e) {}
    public void windowDeiconified(WindowEvent e) {}
    public void windowIconified(WindowEvent e) {}
    public void windowOpened(WindowEvent e) {}

    /*
     **************************************************************************
     * Mouse
     */
    public void mouseClicked(MouseEvent e) {
        /*
         * Double-click, edit a card if we have one.
         */
        if (((e.getModifiers() & InputEvent.BUTTON1_MASK) != 0) &&
            e.getClickCount() >= 2) {

            showCardDialog ();
        }
    }
    public void mouseEntered(MouseEvent e) {}
    public void mouseExited(MouseEvent e) {}
    public void mousePressed(MouseEvent e) {}
    public void mouseReleased(MouseEvent e) {}

    /*
     **************************************************************************
     * Inner collection listener class
     */
    class CollectionListener implements ActionListener {
        Collection collection;
        MainFrame frame;

        CollectionListener (MainFrame inFrame, Collection inCollection) {
            frame = inFrame;
            collection = inCollection;
        }

        public void actionPerformed(ActionEvent e) {
            frame.actionPerformed (new ActionEvent (collection, e.getID(), e.getActionCommand()));
        }
    }
}