/******************************************************************************
 * Module Name : $RCSfile: PreferencesDialog.java,v $
 *
 * Description : Dialog for editing our settings.
 *
 * Updates:
 *   $Log: PreferencesDialog.java,v $
 *   Revision 1.5  2002/07/13 02:22:20  jcouper
 *   job:157
 *   Still tweaking after changing grading and valuation.
 *
 *   Revision 1.4  2000/03/18 00:46:04  Jonathan
 *   Big changes in the way the IndexDialog works, including a new CardDialog.
 *   New preferences for column display.  Permit renaming of title files.
 *
 *   Revision 1.3  2000/02/19 00:18:48  Jonathan
 *   Add saving of settings.
 *
 *   Revision 1.2  2000/01/10 02:22:20  Jonathan
 *   v1.0.0
 *
 *   Revision 1.1  2000/01/09 00:11:55  Jonathan
 *   Edit Dialog, Preferences
 *
 *
 * Copyright Notice:
 * (c) 1999. This source code was developed by Jonathan Couper and friends.
 *            You may use it for any purpose.  No guarantees given or implied.
 *****************************************************************************/
package standup;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.border.*;
import standup.util.*;
import standup.parse.*;
import standup.views.*;

public class PreferencesDialog
    extends JDialog
    implements ActionListener, WindowListener
{
    // Application Settings
    ApplicationSettings applicationSettings = Application.getSettings ();

    // Things on the middle grade preferences panel
    JPanel middlePanel = new JPanel();

    // Grade values panel
    JPanel gradePanel = new JPanel();
    TopToBottomLayout gradeLayout = new TopToBottomLayout();

    // Columns Panel
    JPanel columnsPanel = new JPanel();
    TopToBottomLayout columnsLayout = new TopToBottomLayout();
    JCheckBox showDateCheckbox = new JCheckBox ("Show Date");
    JCheckBox showGradeCheckbox = new JCheckBox ("Show Grade");
    JCheckBox showNmCheckbox = new JCheckBox ("Show NM Value");
    JCheckBox showValueCheckbox = new JCheckBox ("Show Grade Value");
    JCheckBox showInfoCheckbox = new JCheckBox ("Show Info");
    JCheckBox showCommentsCheckbox = new JCheckBox ("Show Comments");

    // Bottom (button) panel
    JPanel buttonPanel = new JPanel();
    JButton doneButton = new JButton("Done");

    Border emptyBorder = new EmptyBorder (5, 5, 5, 5);

    // Keep these so we can save the values later.
    JNumberTextField gradePercentFields[] = new JNumberTextField[(Issue.GRADE_MAX - Issue.GRADE_NONE) + 1];

    /*
     * Creates the dialog panel for this title.
     */
    public PreferencesDialog (Frame frame) {
        super (frame, "Standup Preferences");

        getContentPane().add (middlePanel, BorderLayout.CENTER);
        getContentPane().add (buttonPanel, BorderLayout.SOUTH);

        middlePanel.setLayout (new BorderLayout ());
        middlePanel.add (gradePanel, BorderLayout.CENTER);
        middlePanel.add (columnsPanel, BorderLayout.EAST);

        /*
         * Set up the grades panel.
         */
        gradePanel.setBorder (new CompoundBorder(BorderFactory.createTitledBorder("Grade Values"), emptyBorder));
        gradePanel.setLayout (gradeLayout);
        gradeLayout.setAlignment (TopToBottomLayout.ALIGN_STRETCH);
        gradeLayout.setColumns (3, 2);
        gradeLayout.setHgap (5);
        gradeLayout.setVAlignment (TopToBottomLayout.ALIGN_VCENTER);
        for (int i=Issue.GRADE_NONE; i<=Issue.GRADE_MAX; i++) {
            gradePanel.add (new JLabel (Issue.getNameForGrade (i)));
            gradePercentFields[i - Issue.GRADE_NONE] = new JNumberTextField (0, 200);
            gradePercentFields[i - Issue.GRADE_NONE].setText (String.valueOf (applicationSettings.gradePercentValues[i]));
            gradePanel.add (gradePercentFields[i - Issue.GRADE_NONE]);
            gradePanel.add (new JLabel ("%"));
        }

        /*
         * Set up the columns flags panel.
         */
        columnsPanel.setBorder (new CompoundBorder(BorderFactory.createTitledBorder("Grade Values"), emptyBorder));
        columnsPanel.setLayout (columnsLayout);
        columnsLayout.setAlignment (TopToBottomLayout.ALIGN_STRETCH);
        columnsLayout.setColumns (1, 1);
        columnsLayout.setHgap (5);
        columnsLayout.setVAlignment (TopToBottomLayout.ALIGN_VCENTER);
        columnsPanel.add (showDateCheckbox);
        columnsPanel.add (showGradeCheckbox);
        columnsPanel.add (showNmCheckbox);
        columnsPanel.add (showValueCheckbox);
        columnsPanel.add (showInfoCheckbox);
        columnsPanel.add (showCommentsCheckbox);
        showDateCheckbox.setSelected (applicationSettings.showDateColumn);
        showGradeCheckbox.setSelected (applicationSettings.showGradeColumn);
        showNmCheckbox.setSelected (applicationSettings.showNmValueColumn);
        showValueCheckbox.setSelected (applicationSettings.showValueColumn);
        showInfoCheckbox.setSelected (applicationSettings.showInfoColumn);
        showCommentsCheckbox.setSelected (applicationSettings.showCommentColumn);

        // Bottom (button) panel
        buttonPanel.add (doneButton);

        /*
         * All of our listeners go here.
         */
        doneButton.addActionListener (this);

        /*
         * Validate.  Set preferred size, and centre self.
         */
        pack();

        Dimension pref = this.getPreferredSize();
        setSize(pref.width, pref.height);

        Dimension meSize = getSize();
        Dimension momSize = frame.getSize();
        Point momPos = frame.getLocationOnScreen();

        this.setLocation (momPos.x + (momSize.width / 2) - (meSize.width / 2),
                          momPos.y + (momSize.height / 2) - (meSize.height / 2));
    }

    protected void writeSettings () {
        for (int i=Issue.GRADE_NONE; i<=Issue.GRADE_MAX; i++) {
            applicationSettings.gradePercentValues[i] =
                gradePercentFields[i - Issue.GRADE_NONE].getInt();
        }
        applicationSettings.showDateColumn = showDateCheckbox.isSelected ();
        applicationSettings.showGradeColumn = showGradeCheckbox.isSelected ();
        applicationSettings.showNmValueColumn = showNmCheckbox.isSelected ();
        applicationSettings.showValueColumn = showValueCheckbox.isSelected ();
        applicationSettings.showInfoColumn = showInfoCheckbox.isSelected ();
        applicationSettings.showCommentColumn = showCommentsCheckbox.isSelected ();
    }

    /*
     **************************************************************************
     * LISTENERS BELOW HERE
     **************************************************************************
     */
    /*
     **************************************************************************
     * Act on our button actions.
     */
    public void actionPerformed(ActionEvent e) {
        Object source = e.getSource();

        if (source == doneButton) {
            writeSettings ();
            setVisible (false);
        }
    }

    /*
     **************************************************************************
     * Window listeners
     */
    public void windowActivated(WindowEvent e) {}
    public void windowClosed(WindowEvent e) {}
    public void windowClosing(WindowEvent e) {}
    public void windowDeactivated(WindowEvent e) {}
    public void windowDeiconified(WindowEvent e) {}
    public void windowIconified(WindowEvent e) {}
    public void windowOpened(WindowEvent e) {}
}