/******************************************************************************
 * Module Name : $RCSfile: TitleDialog.java,v $
 *
 * Description : Dialog for editing the issue numbers which go in a title.
 *
 * Updates:
 *   $Log: TitleDialog.java,v $
 *   Revision 1.19  2003/01/23 06:37:46  jcouper
 *   Version 1.1 Changes
 *
 *   Revision 1.18  2002/07/13 01:06:10  jcouper
 *   job:157
 *   Add +/- for grades.  (Also fixed 147 at the same time).
 *
 *   Revision 1.17  2002/07/07 00:54:05  jcouper
 *   job:153
 *   Filenames must be lowercase, and special issues uppercase.
 *
 *   Revision 1.16  2001/10/11 21:43:51  jcouper
 *   134: Constrain filter text to one line and valid chars.  More to come.
 *
 *   Revision 1.15  2001/10/07 03:18:38  jcouper
 *   120: Can now edit HTML text
 *
 *   Revision 1.14  2001/07/11 08:42:28  jcouper
 *   bug:133 When delete an entry, make the next one down selected.
 *
 *   Revision 1.13  2001/04/11 08:22:24  jcouper
 *   Add Filter Support
 *
 *   Revision 1.12  2000/08/23 19:36:12  jcouper
 *   Recognise ENTER key when modifying issue fields.
 *
 *   Revision 1.11  2000/03/18 00:46:04  Jonathan
 *   Big changes in the way the IndexDialog works, including a new CardDialog.
 *   New preferences for column display.  Permit renaming of title files.
 *
 *   Revision 1.10  2000/02/18 22:35:23  Jonathan
 *   Add basic application settings support.
 *
 *   Revision 1.9  2000/01/06 23:46:16  Jonathan
 *   Standup v0.9.3 - Minor Fixes, and report functionality.
 *
 *   Revision 1.8  1999/11/28 07:13:40  Jonathan
 *   Added Flags to Titles, and Polished AllTitlesMatrix report
 *
 *   Revision 1.7  1999/10/24 22:37:54  Jonathan
 *   Proper CVS headers.  Fix A-018/19/20
 *
 *
 * Copyright Notice:
 * (c) 1999. This source code was developed by Jonathan Couper and friends.
 *            You may use it for any purpose.  No guarantees given or implied.
 *****************************************************************************/
package standup;

import java.util.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.border.*;
import standup.parse.*;
import standup.util.*;
import standup.views.*;

public class TitleDialog extends JDialog
    implements ActionListener, ListSelectionListener,
                DocumentListener, WindowListener, FocusListener,
                KeyListener
{
    // The title/ref we are currently manipulating the ranges for.
    TitleRef titleRef = null;
    Title title = null;

    static final int MAX_WIDTH = 640;
    static final int MAX_HEIGHT = 480;

    // Things on the top (title) panel.
    JPanel topPanel = new JPanel();
    // North
    JPanel keyPanel = new JPanel();
    TopToBottomLayout keyLayout = new TopToBottomLayout();
    JLabel nameLabel = new JLabel ("Name:", SwingConstants.RIGHT);
    JLabel fileLabel = new JLabel ("File:", SwingConstants.RIGHT);
    JLimitedTextField nameField = new JLimitedTextField(50, 20);
    JAlphaTextField fileNameField = new JAlphaTextField(20, 60);
    // Center
    JPanel htmlPanel = new JPanel();
    JTextArea htmlText = new JTextArea (4, 40);
    JScrollPane htmlScroll = new JScrollPane (htmlText);

    // Things on the middle (issue) panel
    JPanel middlePanel = new JPanel();
    // Center
    JList rangeList = new JList();
    JScrollPane rangeScrollPane = new JScrollPane(rangeList);
    // East
    JPanel controlPanel = new JPanel();
    ImageIcon addAboveIcon = new ImageIcon (getClass().getResource ("images/add_above.gif"));
    ImageIcon upIcon = new ImageIcon (getClass().getResource ("images/up.gif"));
    ImageIcon deleteIcon = new ImageIcon (getClass().getResource ("images/delete.gif"));
    ImageIcon downIcon = new ImageIcon (getClass().getResource ("images/down.gif"));
    ImageIcon addBelowIcon = new ImageIcon (getClass().getResource ("images/add_below.gif"));
    JButton rangeAboveButton = new JButton (addAboveIcon);
    JButton rangeUpButton = new JButton (upIcon);
    JButton rangeDeleteButton = new JButton (deleteIcon);
    JButton rangeDownButton = new JButton (downIcon);
    JButton rangeBelowButton = new JButton (addBelowIcon);
    // South
    JPanel editPanel = new JPanel();
    ButtonGroup entryButtonGroup = new ButtonGroup();
    // Center
    JPanel editInnerPanel = new JPanel();
    TopToBottomLayout editInnerLayout = new TopToBottomLayout();
    JRadioButton numericEntryButton = new JRadioButton ("Numeric Range");
    JPanel numericRangePanel = new JPanel();
    JNumberTextField startIssueField = new JNumberTextField (4);
    JLabel issueMidLabel = new JLabel ("..");
    JNumberTextField endIssueField = new JNumberTextField (4);
    JRadioButton specialEntryButton = new JRadioButton ("Special Entry");
    JAlphaTextField specialIssueField = new JAlphaTextField (8);
    JPanel applyPanel = new JPanel ();
    JButton applyButton = new JButton ("Apply");

    // South
    TopToBottomLayout filterLayout = new TopToBottomLayout();
    JPanel filterPanel = new JPanel (filterLayout);
    JLabel filterLabel = new JLabel ("Filter Rules");
    JAlphaTextField filterText = new JAlphaTextField(20, 120);

    // Bottom (button) panel
    JPanel buttonPanel = new JPanel();
    JButton doneButton = new JButton("Done");

    Border emptyBorder = new EmptyBorder (5, 5, 5, 5);

    /*
     * Creates the dialog panel for this title.
     */
    public TitleDialog (Dialog dialog, Title newTitle) {
        super (dialog, "Title Dialog", true);
        title = newTitle;
        this.configure (dialog);
    }

    public TitleDialog (Dialog dialog, TitleRef newTitleRef) {
        super (dialog, "TitleRef Dialog", true);
        titleRef = newTitleRef;
        title = titleRef.getTitle ();
        this.configure (dialog);
    }

    /*
     * Creates the dialog panel for this title.
     */
    public TitleDialog (Frame frame, Title newTitle) {
        super (frame, "Title Dialog", true);
        title = newTitle;
        this.configure (frame);
    }

    public TitleDialog (Frame frame, TitleRef newTitleRef) {
        super (frame, "TitleRef Dialog", true);
        titleRef = newTitleRef;
        title = titleRef.getTitle ();
        this.configure (frame);
    }

    protected void configure (Window window) {

        // Lowercase filenames
        fileNameField.setForceLower (true);

        // Uppercase special codes
        specialIssueField.setForceUpper (true);

        getContentPane().add (topPanel, BorderLayout.NORTH);
        getContentPane().add (middlePanel, BorderLayout.CENTER);
        getContentPane().add (buttonPanel, BorderLayout.SOUTH);

        // The top panel
        topPanel.setLayout (new BorderLayout());
        topPanel.add (keyPanel, BorderLayout.NORTH);
        topPanel.add (htmlPanel, BorderLayout.CENTER);

        // Html Panel
        htmlPanel.setBorder (new CompoundBorder(BorderFactory.createTitledBorder("HTML Description"), emptyBorder));
        htmlPanel.add (htmlScroll);

        // The key panel
        keyPanel.setBorder (new CompoundBorder(BorderFactory.createTitledBorder("Detail"), emptyBorder));
        keyPanel.setLayout (keyLayout);
        keyLayout.setAlignment (TopToBottomLayout.ALIGN_STRETCH);
        keyLayout.setColumns (2, 2);
        keyLayout.setHgap (5);
        keyLayout.setVAlignment (TopToBottomLayout.ALIGN_VCENTER);
        keyPanel.add (nameLabel);
        keyPanel.add (nameField);
        keyPanel.add (fileLabel);
        keyPanel.add (fileNameField);

        /*
         * Now add checkboxes for each of the standard title flags.  Selected iff
         * the current title has that flag present.
         */
        for (int i=0; i<Title.templateFlags.length; i++) {
            keyPanel.add (new JLabel (" "));
            String flagName = Title.templateFlags[i];
            JCheckBox flagCheckBox = new JCheckBox (MyStringTokenizer.leadingCaps (Title.templateFlags[i].replace ('_',' ').toLowerCase()));
            flagCheckBox.setSelected (title.hasFlag (flagName));
            flagCheckBox.addActionListener (this);
            keyPanel.add (flagCheckBox);
        }
        for (int i=0; i<Title.userFlags.length; i++) {
            keyPanel.add (new JLabel (" "));
            String flagName = Title.userFlags[i];
            JCheckBox flagCheckBox = new JCheckBox (MyStringTokenizer.leadingCaps (Title.userFlags[i].replace ('_',' ').toLowerCase()));
            flagCheckBox.setSelected (title.hasFlag (flagName));
            flagCheckBox.addActionListener (this);
            keyPanel.add (flagCheckBox);
        }

        // Optional filter panel
        if (titleRef != null) {
            filterText.setExtraAllowed (",");
            filterText.setForceUpper (true);
            filterPanel.add (filterLabel, BorderLayout.NORTH);
            filterPanel.add (filterText, BorderLayout.CENTER);
        }

        // The middle panel
        middlePanel.setBorder (new CompoundBorder(BorderFactory.createTitledBorder("Issues"), emptyBorder));
        middlePanel.setLayout (new BorderLayout());

        middlePanel.add (rangeScrollPane, BorderLayout.CENTER);
        rangeList.setSelectionMode(ListSelectionModel.SINGLE_SELECTION);

        middlePanel.add (controlPanel, BorderLayout.EAST);
        controlPanel.setLayout (new TopToBottomLayout());
        rangeAboveButton.setMargin (new Insets (0, 0, 0, 0));
        rangeUpButton.setMargin (new Insets (0, 0, 0, 0));
        rangeDeleteButton.setMargin (new Insets (0, 0, 0, 0));
        rangeDownButton.setMargin (new Insets (0, 0, 0, 0));
        rangeBelowButton.setMargin (new Insets (0, 0, 0, 0));
        controlPanel.add (rangeAboveButton);
        controlPanel.add (rangeUpButton);
        controlPanel.add (rangeDeleteButton);
        controlPanel.add (rangeDownButton);
        controlPanel.add (rangeBelowButton);

        middlePanel.add (editPanel, BorderLayout.SOUTH);
        editPanel.setLayout (new BorderLayout());
        editPanel.add (editInnerPanel, BorderLayout.CENTER);
        if (titleRef != null) {
            filterText.setText (titleRef.getFilterAsString ());
            editPanel.add (filterPanel, BorderLayout.SOUTH);
        }
        specialIssueField.setForceUpper (true);
        editInnerPanel.setLayout (editInnerLayout);
        editInnerLayout.setAlignment (TopToBottomLayout.ALIGN_STRETCH);
        editInnerLayout.setColumns (2, 1);
        editInnerLayout.setVAlignment (TopToBottomLayout.ALIGN_VCENTER);
        editInnerPanel.add (numericEntryButton);
        editInnerPanel.add (numericRangePanel);
        editInnerPanel.add (specialEntryButton);
        editInnerPanel.add (specialIssueField);
        entryButtonGroup.add(numericEntryButton);
        entryButtonGroup.add(specialEntryButton);
        numericRangePanel.add (startIssueField);
        numericRangePanel.add (issueMidLabel);
        numericRangePanel.add (endIssueField);

        applyPanel.add (applyButton);
        editPanel.add (applyPanel, BorderLayout.EAST);

        // Bottom (button) panel
        buttonPanel.add (doneButton);

        /*
         * All of our listeners go here.
         */
        startIssueField.getDocument().addDocumentListener(this);
        endIssueField.getDocument().addDocumentListener(this);
        specialIssueField.getDocument().addDocumentListener(this);
        nameField.getDocument().addDocumentListener(this);
        fileNameField.getDocument().addDocumentListener(this);
        filterText.getDocument().addDocumentListener(this);
        startIssueField.addFocusListener(this);
        endIssueField.addFocusListener(this);
        specialIssueField.addFocusListener(this);
        htmlText.getDocument().addDocumentListener(this);

        specialEntryButton.addActionListener (this);
        numericEntryButton.addActionListener (this);
        rangeUpButton.addActionListener (this);
        rangeAboveButton.addActionListener (this);
        rangeDeleteButton.addActionListener (this);
        rangeBelowButton.addActionListener (this);
        rangeDownButton.addActionListener (this);
        applyButton.addActionListener (this);
        doneButton.addActionListener (this);
        startIssueField.addKeyListener (this);
        endIssueField.addKeyListener (this);
        specialIssueField.addKeyListener (this);
        rangeList.addListSelectionListener(this);

        /*
         * All of our state initialisation goes here.
         */
        filterText.setEnabled (true);
        rangeUpButton.setEnabled (false);
        rangeAboveButton.setEnabled (false);
        rangeDeleteButton.setEnabled (false);
        rangeBelowButton.setEnabled (false);
        rangeDownButton.setEnabled (false);

        /*
         * Configure our controls.
         */
        rangeUpButton.setEnabled (false);
        rangeAboveButton.setEnabled (true);
        rangeDeleteButton.setEnabled (false);
        rangeBelowButton.setEnabled (true);
        rangeDownButton.setEnabled (false);

        rangeList.setListData (title);
        rangeList.clearSelection ();

        /*
         * Other text parameters to be edited.
         */
        fileNameField.setText (title.getKeyname());

        String defaultName = title.getDefaultTitleName ();
        if ((defaultName != null) && !defaultName.equals ("")) {
            nameField.setText (defaultName);

        } else {
            nameField.setText (MyStringTokenizer.leadingCaps (title.getKeyname().replace('_',' ')));
        }

        htmlText.setText (title.getHtmlDescription());

        enableUpdateControls ();

        /*
         * Set preferred size, and centre self.
         */
        Dimension pref = this.getPreferredSize();

        if (pref.width > MAX_WIDTH) { pref.width = MAX_WIDTH; }
        if (pref.height > MAX_HEIGHT) { pref.height = MAX_HEIGHT; }

        setSize (pref.width, pref.height);
        pack ();

        Dimension meSize = getSize();
        Dimension momSize = window.getSize();
        Point momPos = window.getLocationOnScreen();

        int myX = momPos.x + (momSize.width / 2) - (meSize.width / 2);
        int myY = momPos.y + (momSize.height / 2) - (meSize.height / 2);

        if (myX < 0) myX = 0;
        if (myY < 0) myY = 0;

        this.setLocation (myX, myY);
    }

    /*
     * Determine if the update button is allowed or not.
     */
    public boolean isUpdateAllowed () {
        /*
         * Special entry.  This is easy, as long as it's not a duplicate,
         * and not numeric, and not null.
         */
        if (specialEntryButton.isSelected()) {
            String specialCode = specialIssueField.getText();
            if (specialCode.equals("")) {
                return false;
            }
            try {
                Integer codeAsInt = new Integer (specialCode);
                return false;
            }
            catch (NumberFormatException nfe) {
            }
            if (title.getIssueByCode (specialCode) != null) {
                return false;
            }
            return true;

        /*
         * Numeric entry.  Simple test, are they both valid numbers with (a < b)?
         */
        } else {
            if (!startIssueField.isNumeric()) {
                return false;
            }
            if (!endIssueField.isNumeric()) {
                return false;
            }
            int startNumber = startIssueField.getInt();
            int endNumber = endIssueField.getInt();
            if (endNumber < startNumber) {
                return false;
            }

            /*
             * Now we just want to make sure that if we replaced this range with the
             * proposed range, would we be causing a duplication?
             */
            int pos = rangeList.getSelectedIndex();
            IssueRange selectedRange = (IssueRange) title.elementAt (pos);

            for (Enumeration e = title.elements() ; e.hasMoreElements() ;) {
                IssueRange issueRange = (IssueRange) e.nextElement();
                if (issueRange != selectedRange) {
                    if (issueRange.isOverlap (startNumber, endNumber)) {
                        return false;
                    }
                }
            }
            return true;
        }
    }

    public void enableUpdateControls () {
        /*
         * The move up/down and delete buttons only work if we have a selection.
         */
        int pos = rangeList.getSelectedIndex();
        boolean selected = (pos >= 0);
        rangeUpButton.setEnabled (selected && (pos > 0));
        rangeAboveButton.setEnabled (true);
        rangeDeleteButton.setEnabled (selected);
        rangeBelowButton.setEnabled (true);
        rangeDownButton.setEnabled (selected && (pos < (rangeList.getModel().getSize() - 1)));

        /*
         * Selected?  Enable fields and buttons, set initial values, etc, etc.
         */
        if (selected) {
            IssueRange issueRange = (IssueRange) rangeList.getSelectedValue ();
            boolean prevCodeIsNumeric = true;

            /*
             * Empty range?  Can be either.  Depends on previous.
             */
            if (issueRange.size() <= 0) {
            // Go back and find previous defined code, look if it is numeric or not.
                for (int prev=pos-1; prev>=0;  prev--) {
                    IssueRange prevRange = (IssueRange) rangeList.getModel().getElementAt (prev);
                    Issue prevStartIssue = prevRange.getStartIssue();
                    if (prevStartIssue != null) {
                        prevCodeIsNumeric = prevStartIssue.isCodeNumeric();
                        break;
                    }
                }

                numericEntryButton.setSelected (prevCodeIsNumeric);
                numericEntryButton.setEnabled  (true);
                specialEntryButton.setSelected (!prevCodeIsNumeric);
                specialEntryButton.setEnabled  (true);
                startIssueField.setText ("");
                startIssueField.setEnabled  (true);
                endIssueField.setText ("");
                endIssueField.setEnabled  (true);
                specialIssueField.setText ("");
                specialIssueField.setEnabled  (true);

                if (prevCodeIsNumeric) {
                    startIssueField.requestFocus();
                } else {
                    specialIssueField.requestFocus();
                }
            /*
             * Numeric range?  That's fine, force them to use the range entry.
             */
            } else if (issueRange.getStartIssue().isCodeNumeric()) {
                numericEntryButton.setSelected (true);
                numericEntryButton.setEnabled  (true);
                specialEntryButton.setSelected (false);
                specialEntryButton.setEnabled  (false);
                startIssueField.setText (issueRange.getStartIssue().getCode());
                startIssueField.setEnabled  (true);
                endIssueField.setText (issueRange.getEndIssue().getCode());
                endIssueField.setEnabled  (true);
                endIssueField.requestFocus();
                endIssueField.setSelectionStart(0);
                endIssueField.setSelectionEnd(endIssueField.getText().length());
                specialIssueField.setText ("");
                specialIssueField.setEnabled  (false);
            /*
             * Non-numeric.  We need specials, then.
             */
            } else {
                numericEntryButton.setSelected (false);
                numericEntryButton.setEnabled  (false);
                specialEntryButton.setSelected (true);
                specialEntryButton.setEnabled  (true);
                startIssueField.setText ("");
                startIssueField.setEnabled  (false);
                endIssueField.setText ("");
                endIssueField.setEnabled  (false);
                specialIssueField.setText (issueRange.getStartIssue().getCode());
                specialIssueField.setEnabled  (true);
                specialIssueField.requestFocus();
                specialIssueField.setSelectionStart(0);
                specialIssueField.setSelectionEnd(endIssueField.getText().length());
            }

        /*
         * Deselected?  Clear it all.
         */
        } else {
            numericEntryButton.setSelected (true);
            numericEntryButton.setEnabled  (false);
            specialEntryButton.setSelected (false);
            specialEntryButton.setEnabled  (false);
            startIssueField.setText ("");
            startIssueField.setEnabled  (false);
            endIssueField.setText ("");
            endIssueField.setEnabled  (false);
            specialIssueField.setText ("");
            specialIssueField.setEnabled  (false);
        }

        applyButton.setEnabled (isUpdateAllowed ());
    }

    /*
     * The user has changed a numeric or special range, deal with it.  This
     * must be valid, the buttons won't allow otherwise.
     */
    public void doRangeUpdate () {
        int pos = rangeList.getSelectedIndex();
        IssueRange selectedRange = (IssueRange) title.elementAt (pos);

        // rangeList.clearSelection();

        /*
         * Special entry.  This is simpler, so we'll do this case first.
         * It either creates a single entry range (if there was nothing)
         * or replaces the range.
         */
        if (specialEntryButton.isSelected()) {
            Issue firstIssue = selectedRange.getStartIssue ();
            if (firstIssue == null) {
                selectedRange.addElement (new Issue (specialIssueField.getText()));

            } else {
                firstIssue.setCode (specialIssueField.getText());
            }
            rangeList.repaint();

        /*
         * Must be a numeric range selection.  Ouch.  OK, got to be done.
         */
        } else {
            int startNumber = startIssueField.getInt();
            int endNumber = endIssueField.getInt();
            IssueRange newRange = new IssueRange();

            /*
             * Build our new range first.  Salvage any issues from the existing
             * range if possible.
             */
            for (int i= startNumber; i<=endNumber; i++) {
                String code = String.valueOf (i);

                Issue issue = title.getIssueByCode (code);
                if (issue == null) {
                    issue = new Issue (code);
                }
                newRange.addElement (issue);
            }

            /*
             * Remove the old list and replace with the new one.
             */
            title.setElementAt (newRange, pos);
            rangeList.repaint();
        }

        /*
         * Clear everything up.
         */
        title.flagTemplateDataModified();
    }

    /*
     **************************************************************************
     * LISTENERS BELOW HERE
     **************************************************************************
     */
    /*
     **************************************************************************
     * List listeners.
     */
    public void valueChanged(ListSelectionEvent e) {
        /*
         * Range list (de)selected?  We have some work to do either way.
         */
        if (e.getSource() == rangeList) {
            enableUpdateControls ();
        }
    }

    /*
     **************************************************************************
     * Act on our button actions.
     */
    public void actionPerformed(ActionEvent e) {
        Object source = e.getSource();

        if (source == doneButton) {
            setVisible (false);

        /* Apply Range Changes? */
        } else if (source == applyButton) {
            doRangeUpdate ();

        /*
         * Move up/down?  Swap with element above/below.  We know this is safe, because we
         * masked buttons to ensure it.
         */
        } else if (source == rangeUpButton) {
            int pos = rangeList.getSelectedIndex();

            IssueRange selectedRange = (IssueRange) title.elementAt (pos);
            title.setElementAt ((IssueRange) title.elementAt (pos-1), pos);
            title.setElementAt (selectedRange, pos-1);
            title.flagTemplateDataModified();

            /*
             * The swap is complete on the title.  Reset the list.
             */
            rangeList.setListData (title);
            rangeList.setSelectedValue (selectedRange, true);

        } else if (source == rangeAboveButton) {
            IssueRange newRange = new IssueRange();
            int pos = rangeList.getSelectedIndex();

            // If nothing selected, insert at top of list.
            if (pos < 0) {
                pos = 0;
            }
            title.insertElementAt (newRange, pos);
            title.flagTemplateDataModified();

            rangeList.setListData (title);
            rangeList.setSelectedValue (newRange, true);

        } else if (source == rangeDeleteButton) {
            int pos = rangeList.getSelectedIndex();

            title.removeElementAt (pos);
            title.flagTemplateDataModified();

            rangeList.setListData (title);
            if (pos >= title.size ()) {
                rangeList.clearSelection ();
            } else {
                rangeList.setSelectedIndex (pos);
            }

        } else if (source == rangeBelowButton) {
            IssueRange newRange = new IssueRange();
            int pos = rangeList.getSelectedIndex();

            // If nothing selected, insert at end of list.
            if (pos < 0) {
                title.addElement (newRange);
                title.flagTemplateDataModified();

            } else {
                title.insertElementAt (newRange, pos+1);
                title.flagTemplateDataModified();
            }
            rangeList.setListData (title);
            rangeList.setSelectedValue (newRange, true);

        } else if (source == rangeDownButton) {
            int pos = rangeList.getSelectedIndex();

            IssueRange selectedRange = (IssueRange) title.elementAt (pos);
            title.setElementAt ((IssueRange) title.elementAt (pos+1), pos);
            title.setElementAt (selectedRange, pos+1);
            title.flagTemplateDataModified();

            rangeList.setListData (title);
            rangeList.setSelectedValue (selectedRange, true);

        /*
         * Our radio buttons.  We only have a choice (i.e. they are only enabled)
         * when the range is originally empty.  Hence our action on button change
         * is simple.
         */
        } else if (source == numericEntryButton) {
            startIssueField.setText ("");
            endIssueField.setText ("");
            specialIssueField.setText ("");
            startIssueField.requestFocus();

        } else if (source == specialEntryButton) {
            startIssueField.setText ("");
            endIssueField.setText ("");
            specialIssueField.setText ("");
            specialIssueField.requestFocus();

        } else if (source instanceof JCheckBox) {
            JCheckBox flagCheckBox = (JCheckBox) source;

            if (flagCheckBox.isSelected ()) {
                title.setFlag (flagCheckBox.getText().replace (' ','_').toUpperCase());
            } else {
                title.clearFlag (flagCheckBox.getText().replace (' ','_').toUpperCase());
            }
        }
    }

    /*
     **************************************************************************
     * Act on document changes.
     */
    public void changedUpdate (DocumentEvent e) {
        documentUpdate (e);
    }
    public void insertUpdate (DocumentEvent e) {
        documentUpdate (e);
    }
    public void removeUpdate (DocumentEvent e) {
        documentUpdate (e);
    }
    public void documentUpdate (DocumentEvent e) {
        Document source = e.getDocument();

        if ((source == startIssueField.getDocument()) ||
            (source == endIssueField.getDocument()) ||
            (source == specialIssueField.getDocument())) {

            applyButton.setEnabled (isUpdateAllowed ());

        } else if (source == nameField.getDocument()) {
            if (title != null) {
                title.setDefaultTitleName (nameField.getText ());
            }

        } else if (source == htmlText.getDocument()) {
            if (title != null) {
                title.setHtmlDescription (htmlText.getText ());
            }

        } else if (source == fileNameField.getDocument()) {
            if (title != null) {
                title.markForRename (fileNameField.getText ());
            }

        } else if (source == filterText.getDocument()) {
            if (titleRef != null) {
                titleRef.setFilterFromString (filterText.getText ());
                titleRef.getIndex().setModified (true);
            }
        }
    }

    /*
     **************************************************************************
     * Key listeners
     */
    public void keyPressed(KeyEvent e) {
        if (e.getKeyCode() == KeyEvent.VK_ENTER) {
            doRangeUpdate ();
        }
    }
    public void keyReleased(KeyEvent e) {}
    public void keyTyped(KeyEvent e) {}

    /*
     **************************************************************************
     * Focus listeners
     */
    public void focusGained(FocusEvent e) {
        Object source = e.getSource();
        if (source == specialIssueField) {
            specialEntryButton.setSelected (true);
            startIssueField.setText ("");
            endIssueField.setText ("");

        } else if ((source == startIssueField) || (source == endIssueField)) {
            numericEntryButton.setSelected (true);
            specialIssueField.setText ("");
        }
    }
    public void focusLost(FocusEvent e) {}

    /*
     **************************************************************************
     * Window listeners
     */
    public void windowActivated(WindowEvent e) {}
    public void windowClosed(WindowEvent e) {}
    public void windowClosing(WindowEvent e) {}
    public void windowDeactivated(WindowEvent e) {}
    public void windowDeiconified(WindowEvent e) {}
    public void windowIconified(WindowEvent e) {}
    public void windowOpened(WindowEvent e) {}
}