/******************************************************************************
 * Module Name : $RCSfile: Card.java,v $
 *
 * Description : Card, belonging to a collection, contains title references.
 *                All cards are created at startup, when index.dat is read.
 *
 * Note: Vector of TitleRef objects.
 *
 * Updates:
 *   $Log: Card.java,v $
 *   Revision 1.7  2001/10/07 04:42:27  jcouper
 *   141: Now prompts for confirm removing filtered title ref
 *
 *   Revision 1.6  2001/10/07 03:57:35  jcouper
 *   121: Added up/down buttons for titleRef
 *
 *   Revision 1.5  2000/03/18 00:46:13  Jonathan
 *   Big changes in the way the IndexDialog works, including a new CardDialog.
 *   New preferences for column display.  Permit renaming of title files.
 *
 *   Revision 1.4  1999/11/27 09:56:58  Jonathan
 *   Add filtering placeholders.  Doesn't quite do filtering, but it's getting
 *   jolly close.
 *
 *   Revision 1.3  1999/10/24 22:37:55  Jonathan
 *   Proper CVS headers.  Fix A-018/19/20
 *
 *
 * Copyright Notice:
 * (c) 1999. This source code was developed by Jonathan Couper and friends.
 *            You may use it for any purpose.  No guarantees given or implied.
 *****************************************************************************/
package standup.parse;

import java.util.*;

public class Card extends Vector {
    String name = "";
    Object renderer = null;

    /*
     **************************************************************************
     * Constructors.
     **************************************************************************
     */
    public Card (String newName) {
        this.name = newName;
    }

    /*
     **************************************************************************
     * Override our inherited methods.
     **************************************************************************
     */
    public void addElement (Object object) {
        if (! (object instanceof TitleRef)) {
            System.out.println ("Card is Vector of TitleRef");
            System.exit (1);
        }
        super.addElement (object);
    }
    public Object set(int index, Object element) {
        if (! (element instanceof TitleRef)) {
            System.out.println ("Card is Vector of TitleRef");
            System.exit (1);
        }
        return super.set (index, element);
    }
    public void setElementAt(Object obj, int index) {
        if (! (obj instanceof TitleRef)) {
            System.out.println ("Card is Vector of TitleRef");
            System.exit (1);
        }
        super.setElementAt (obj, index);
    }

    public String toString () {
        return getName();
    }

    /*
     **************************************************************************
     * Get and set methods.
     **************************************************************************
     */
    public String getName () {
        return name;
    }

    public void setName (String newName) {
        name = newName;
    }

    /*
     * Return the total number of issues on this card.
     */
    public int getTotalIssueCount () {
        int totalIssues = 0;

        for (Enumeration e = elements() ; e.hasMoreElements() ;) {
            TitleRef titleRef = (TitleRef) e.nextElement();
            totalIssues = totalIssues + titleRef.numIssues();
        }

        return totalIssues;
    }

    /*
     * Find the corresponding title, and the issue number within the
     * title.
     */
    public Issue issueAtArrayPos (int pos) {
        TitleRef titleRef = null;
        int jumpedIssues = 0;
        int issueOffset = 0;

        if ((pos < 0) || (pos > getTotalIssueCount())) {
            return null;
        }

       for (Enumeration e = elements() ; e.hasMoreElements() ;) {
            titleRef = (TitleRef) e.nextElement();

            issueOffset = pos - jumpedIssues;
            if (issueOffset < titleRef.numIssues()) {
                break;
            }
            jumpedIssues = jumpedIssues + titleRef.numIssues();
       }
       return titleRef.getIssueByArrayPos(issueOffset);
    }

    public TitleRef titleRefAtArrayPos (int pos) {
        TitleRef titleRef = null;
        int jumpedIssues = 0;
        int issueOffset = 0;

        if ((pos < 0) || (pos > getTotalIssueCount())) {
            return null;
        }

       for (Enumeration e = elements() ; e.hasMoreElements() ;) {
            titleRef = (TitleRef) e.nextElement();

            issueOffset = pos - jumpedIssues;
            if (issueOffset < titleRef.numIssues()) {
                break;
            }
            jumpedIssues = jumpedIssues + titleRef.numIssues();
       }
       return titleRef;
    }

    /*
     * This is like contains(), but we want to know if one of our title refs
     * is to the specified title.
     */
    public TitleRef cardReferenceTo (Title title) {
       for (Enumeration e = elements() ; e.hasMoreElements() ;) {
            TitleRef titleRef = (TitleRef) e.nextElement();
            if (titleRef.getTitle() == title) {
                return titleRef;
            }
       }
        return null;
    }
}