/******************************************************************************
 * Module Name : $RCSfile: Collection.java,v $
 *
 * Description : Vector of index cards which represents a collection
 * 				  within the entire swag of comics.
 *
 * Note: Vector of Card objects.
 *
 * Updates:
 *   $Log: Collection.java,v $
 *   Revision 1.3  2000/03/18 00:46:13  Jonathan
 *   Big changes in the way the IndexDialog works, including a new CardDialog.
 *   New preferences for column display.  Permit renaming of title files.
 *
 *   Revision 1.2  1999/10/24 22:37:55  Jonathan
 *   Proper CVS headers.  Fix A-018/19/20
 *
 *
 * Copyright Notice:
 * (c) 1999. This source code was developed by Jonathan Couper and friends. 
 * 	      	  You may use it for any purpose.  No guarantees given or implied.
 *****************************************************************************/ 
package standup.parse;

import java.util.*;

public class Collection extends Vector {
	String name = null;
	Index index = null;	// Backwards link to parent
	boolean modified = false;
	
    /*
	 **************************************************************************
	 * Constructors.
	 **************************************************************************
	 */
	public Collection (Index newIndex, String newName) {
		index = newIndex;
		name = newName;		
	}
	
   /*
	 **************************************************************************
	 * Override our inherited methods.
	 **************************************************************************
	 */
	public void addElement (Object object) {
		if (! (object instanceof Card)) {
			System.out.println ("Collection is Vector of Card");
			System.exit (1);
		}
		super.addElement (object);
	}	

	public String toString () {
		return getName();
	}
	
    /*
	 **************************************************************************
	 * Get and set methods.
	 **************************************************************************
	 */
	public String getName () {
		return name;
	}
	
	public void setName (String newName) {
		name = newName;
	}
	
	public void setModified (boolean newModified) {
		modified = newModified;
	}
	
	public boolean getModified () {
		return modified;
	}
	
	public Index getIndex () {
		return index;
	}
	
    /*
	 **************************************************************************
	 * Useful methods.
	 **************************************************************************
	 */
	/*
	 * Good old bubble sort of cards by getName(), A..Z.
	 */
	public void sortCardsByName () {
		boolean sorting = true;
		while (sorting) {
			sorting = false;
			
			for (int i=0; i<size()-1; i++) {
				Card firstCard = (Card) elementAt (i);
				Card secondCard = (Card) elementAt (i+1);
				
				if (firstCard.getName().compareToIgnoreCase(secondCard.getName()) > 0) {
					setElementAt (secondCard, i);
					setElementAt (firstCard, i+1);
					sorting = true;
				}
			}
		}
	}
}