/******************************************************************************
 * Module Name : $RCSfile: AllTitleSummary.java,v $
 *
 * Description : Generates report of all titles in an index.
 *
 * Updates:
 *   $Log: AllTitleSummary.java,v $
 *   Revision 1.11  2003/01/23 06:37:46  jcouper
 *   Version 1.1 Changes
 *
 *   Revision 1.10  2002/07/13 02:22:35  jcouper
 *   job:157
 *   Still tweaking after changing grading and valuation.
 *
 *   Revision 1.9  2002/07/10 07:32:34  jcouper
 *   job:158
 *   Option to only report on held titles.
 *
 *   Revision 1.8  2002/07/07 00:51:09  jcouper
 *   job:151
 *   Show partial valuation even if not all comics valued.
 *
 *   Revision 1.7  2001/07/09 01:32:21  jcouper
 *   bug:132 Fix Report problems
 *
 *   Revision 1.6  2000/08/23 19:41:20  jcouper
 *   Sort titles before reporting.
 *
 *   Revision 1.5  2000/03/18 00:46:46  Jonathan
 *   Report enhancements to reduce size of long runs with the same grade.
 *
 *   Revision 1.4  2000/02/18 22:35:47  Jonathan
 *   Display reports in HTML.
 *
 *   Revision 1.3  2000/01/09 00:11:55  Jonathan
 *   Edit Dialog, Preferences
 *
 *   Revision 1.2  1999/11/27 09:55:02  Jonathan
 *   Added a new report function.  Also some structural changes, create report
 *   objects rather than cheating with static methods.  Use Reflection to generate
 *   reports.
 *
 *   Revision 1.1  1999/11/15 23:22:15  Jonathan
 *   Rename for clarity
 *
 *   Revision 1.1  1999/11/14 04:32:24  Jonathan
 *   Basic reports
 *
 *
 * Copyright Notice:
 * (c) 1999. This source code was developed by Jonathan Couper and friends.
 *            You may use it for any purpose.  No guarantees given or implied.
 *****************************************************************************/
package standup.report;

import java.util.*;
import standup.parse.*;

public class AllTitleSummary extends AbstractReport
{
    // Keys for our report flavours
    public static final int ALL_TITLES = 0;
    public static final int HELD_TITLES = 0;

    // Default name is generated from a report instance.
    public String getDefaultName () {
        return "title_summary.html";
    }

    public StringBuffer generateHtml (Index index, Integer flavour) {
        int intFlavor = flavour.intValue();
        StringBuffer buf = new StringBuffer ();
        Vector allTitles = index.getAllTitles();
        int totalIssues = 0;
        int totalIssuesHeld = 0;
        double totalGradeValue = 0.0;
        int totalNumHeldAndValued = 0;

        index.sortAllTitlesByGetAnyName();

        buf.append (tagHtmlStart());
        buf.append (tagHead(index.toString()));
        buf.append (tagBodyStart());
        buf.append (tagTableStart());

        buf.append ("<tr>\n");
        buf.append ("<th>Title</th>\n");
        buf.append ("<th>#Issues</th>\n");
        buf.append ("<th>#Held</th>\n");
        buf.append ("<th>#Valued</th>\n");
        buf.append ("<th>Graded Value</th>\n");
        buf.append ("</tr>\n");

        for (Enumeration e = allTitles.elements() ; e.hasMoreElements() ;) {
            Title title = (Title) e.nextElement();

            // Number of issues, held or not, no filtering
            int numIssues = title.numIssues();

            // Number of issues, held, no filtering
            TitleRef heldTitleRef = new TitleRef (index, title);
            heldTitleRef.setFilterHeldOnly(true);
            int numIssuesHeld = title.numIssues(heldTitleRef);

            if ((intFlavor == HELD_TITLES) && (numIssuesHeld == 0)) {
                continue;
            }

            // How many are held and valued?
            int numHeldAndValued = title.numValued (heldTitleRef);

            // Total value of all held?
            double gradeValue = title.getGradeValue(heldTitleRef);

            buf.append ("<tr>\n");
            buf.append ("<td>" + title.getAnyName() + "</td>\n");
            buf.append ("<td>" + numIssues + "</td>\n");
            buf.append ("<td>" + numIssuesHeld + "</td>\n");
            buf.append ("<td>" + numHeldAndValued + "</td>\n");
            buf.append ("<td>");
            if (gradeValue == 0.0) {
                buf.append ("&nbsp;");

            } else {
                buf.append ("$" + ((int) gradeValue) + ".00");
            }
            if (numHeldAndValued < numIssuesHeld) {
                buf.append (" <b>[*]</b>");
            }
            buf.append ("</td>\n");

            totalIssues = totalIssues + numIssues;
            totalIssuesHeld = totalIssuesHeld + numIssuesHeld;
            totalGradeValue = totalGradeValue + gradeValue;
            totalNumHeldAndValued = totalNumHeldAndValued + numHeldAndValued;

            buf.append ("</tr>\n");
        }

        buf.append ("<tr>\n");
        buf.append ("<td><b>Total Counts</b></td>\n");
        buf.append ("<td>" + totalIssues + "</td>\n");
        buf.append ("<td>" + totalIssuesHeld + "</td>\n");
        buf.append ("<td>" + totalNumHeldAndValued + "</td>\n");
        buf.append ("<td>$" + ((int) totalGradeValue) + ".00");
        if (totalNumHeldAndValued < totalIssuesHeld) {
            buf.append (" <b>[*]</b>");
        }
        buf.append ("</td>\n");
        buf.append ("</tr>\n");

        buf.append (tagTableEnd());
        buf.append (tagBodyEnd());
        buf.append (tagHtmlEnd());

        return buf;
    }
}