/******************************************************************************
 * Module Name : $RCSfile: InsuranceDetails.java,v $
 *
 * Description : Generates matrix report of all titles in an index.
 *
 * Updates:
 *
 * Copyright Notice:
 * (c) 1999. This source code was developed by Jonathan Couper and friends.
 *            You may use it for any purpose.  No guarantees given or implied.
 *****************************************************************************/
package standup.report;

import java.util.*;
import standup.parse.*;

public class InsuranceDetails extends AbstractReport
{
    // Keys for our report flavours
    public static final int DEFAULT_FORMAT = 0;

    // Report values over this amount
    public static final double REPORT_VALUE = 10.0;

    public String getDefaultName () {
        return "insurance_details.html";
    }

    public StringBuffer generateHtml (Index index, Integer flavour) {
        StringBuffer buf = new StringBuffer ();
        Vector allTitles = index.getAllTitles();
        int totalIssues = 0;
        int totalIssuesHeld = 0;
        int rowCount = 0;

        double totalValue = 0.0;

        index.sortAllTitlesByGetAnyName();

        buf.append (tagHtmlStart());
        buf.append (tagHead(index.toString()));
        buf.append (tagBodyStart());
        buf.append (tagTableStart());

        for (Enumeration e = allTitles.elements() ; e.hasMoreElements() ;) {
            Title title = (Title) e.nextElement();

            String titleName = title.getDefaultTitleName();

            /*
             * Get the number of issues, with no filtering, held and total.
             */
            TitleRef heldTitleRef = new TitleRef (index, title);
            heldTitleRef.setFilterHeldOnly(true);

            int numHeld = title.numIssues (heldTitleRef);
            double valueHeld = title.getGradeValue (heldTitleRef);

            totalValue = totalValue + valueHeld;

            /*
             * We have no issues?  e.g. we're doing held only, and have none.
             */
            if (numHeld == 0) {
                continue;

            /*
             * Other case, Title, #Held, Values one line, then a big block
             * with a summary.
             */
            } else {
                buf.append (tagTrStart ());
                buf.append ("<TD><b>" + titleName + "</b></TD>");
                buf.append ("<TD>Hold " + numHeld + "</TD>");
                buf.append ("<TD>Total " + Issue.valueToString (valueHeld) + "</TD>");
                buf.append (tagTrEnd ());

                buf.append (tagTrStart ());
                buf.append ("<TD colspan=3>" + title.getRunList (true, true) + "</TD");
                buf.append (tagTrEnd ());
            }
        }

        // total
        buf.append (tagTrStart ());
        buf.append ("<TD colspan=3>Grand Total = " + Issue.valueToString (totalValue) + "</TD");
        buf.append (tagTrEnd ());

        buf.append (tagTableEnd());
        buf.append (tagBodyEnd());
        buf.append (tagHtmlEnd());

        return buf;
    }
}