/******************************************************************************
 * Module Name : $RCSfile: ReportDialog.java,v $
 *
 * Description : Dialog for generating any report.
 *
 * Updates:
 *   $Log: ReportDialog.java,v $
 *   Revision 1.12  2003/01/23 06:37:46  jcouper
 *   Version 1.1 Changes
 *
 *   Revision 1.11  2002/07/10 07:32:34  jcouper
 *   job:158
 *   Option to only report on held titles.
 *
 *   Revision 1.10  2001/07/09 01:32:21  jcouper
 *   bug:132 Fix Report problems
 *
 *   Revision 1.9  2000/03/18 00:46:46  Jonathan
 *   Report enhancements to reduce size of long runs with the same grade.
 *
 *   Revision 1.8  2000/02/18 22:35:47  Jonathan
 *   Display reports in HTML.
 *
 *   Revision 1.7  2000/01/10 02:22:43  Jonathan
 *   Cleaned up layout.
 *
 *   Revision 1.6  2000/01/09 00:11:55  Jonathan
 *   Edit Dialog, Preferences
 *
 *   Revision 1.5  1999/11/27 09:55:02  Jonathan
 *   Added a new report function.  Also some structural changes, create report
 *   objects rather than cheating with static methods.  Use Reflection to generate
 *   reports.
 *
 *   Revision 1.4  1999/11/15 23:22:15  Jonathan
 *   Rename for clarity
 *
 *   Revision 1.3  1999/11/15 22:27:02  Jonathan
 *   Report stuff compiles at least.  Doesn't run, mind you.
 *
 *   Revision 1.2  1999/11/14 04:32:24  Jonathan
 *   Basic reports
 *
 *   Revision 1.1  1999/11/14 03:11:44  Jonathan
 *   Report Placeholder
 *
 *
 * Copyright Notice:
 * (c) 1999. This source code was developed by Jonathan Couper and friends.
 *            You may use it for any purpose.  No guarantees given or implied.
 *****************************************************************************/
package standup.report;

import java.util.Enumeration;
import java.util.Vector;
import java.lang.reflect.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.text.*;
import javax.swing.border.*;
import javax.swing.filechooser.FileFilter;
import standup.parse.*;

public class ReportDialog extends JDialog
    implements ActionListener
{
    public static String reportClasses[] = {
        "standup.report.AllTitleSummary",
        "standup.report.AllTitleSummary",
        "standup.report.AllTitleMatrix",
        "standup.report.InsuranceDetails"
    };
    public static String reportDescriptions[] = {
        "Title Summary (All)",
        "Title Summary (Held Only)",
        "Title Matrix",
        "Insurance Details"
    };
    public static int reportFlavours[] = {
        AllTitleSummary.ALL_TITLES,
        AllTitleSummary.HELD_TITLES,
        AllTitleMatrix.TITLE_MATRIX,
        InsuranceDetails.DEFAULT_FORMAT
    };

    // The index we are reporting on.
    Index index = null;
    // The current report.
    AbstractReport currentReport = null;
    // The output we make from it
    StringBuffer reportContent = null;
    // Our temp file where we kept the output.
    File tempFile = null;

    // Things on the top (title) panel.
    JPanel topPanel = new JPanel();
    JComboBox scriptCombo = new JComboBox();

    // Things on the middle (issue) panel
    JPanel middlePanel = new JPanel();
    JEditorPane outputEditorPane = new JEditorPane();
    JScrollPane outputPane = new JScrollPane (outputEditorPane);

    // Bottom (button) panel
    JPanel buttonPanel = new JPanel();
    JButton generateButton = new JButton("Generate");
    JButton saveAsButton = new JButton("Save As...");
    JButton displayButton = new JButton("Display");
    JButton doneButton = new JButton("Done");

    /*
     * Creates the dialog panel for this title.
     */
    public ReportDialog (Dialog dialog, Index newIndex) {
        super (dialog, "Report Dialog", true);
        this.configure (dialog, newIndex);
    }

    /*
     * Creates the dialog panel for this title.
     */
    public ReportDialog (Frame frame, Index newIndex) {
        super (frame, "Report Dialog", true);
        this.configure (frame, newIndex);
    }

    protected void configure (Window window, Index newIndex) {
        index = newIndex;

        getContentPane().add (topPanel, BorderLayout.NORTH);
        getContentPane().add (middlePanel, BorderLayout.CENTER);
        getContentPane().add (buttonPanel, BorderLayout.SOUTH);

        // The top panel
        topPanel.setBorder (BorderFactory.createEmptyBorder (5, 5, 5, 5));
        topPanel.setLayout (new FlowLayout ());
        topPanel.add (scriptCombo, BorderLayout.CENTER);

        // The middle panel
        outputPane.setSize (400, 300);
        middlePanel.setBorder (BorderFactory.createEmptyBorder (5, 5, 5, 5));
        middlePanel.setLayout (new BorderLayout());
        middlePanel.add (outputPane);

        // Bottom (button) panel
        buttonPanel.add (generateButton);
        buttonPanel.add (saveAsButton);
        buttonPanel.add (displayButton);
        buttonPanel.add (doneButton);

        /*
         * All of our listeners go here.
         */
        generateButton.addActionListener(this);
        saveAsButton.addActionListener(this);
        displayButton.addActionListener(this);
        doneButton.addActionListener(this);

        /*
         * Configure our controls.
         */
        displayButton.setEnabled (false);
        saveAsButton.setEnabled (false);

        Class getDescriptionsParams[] = {};
        Object getDescriptionsArgs[] = {};

        for (int i=0; i<reportDescriptions.length; i++) {
            scriptCombo.addItem (reportDescriptions[i]);
        }

        /*
         * Set preferred size, and centre self.
         */
        setSize (600, 400);

        Dimension meSize = getSize();
        Dimension momSize = window.getSize();
        Point momPos = window.getLocationOnScreen();

        this.setLocation (momPos.x + (momSize.width / 2) - (meSize.width / 2),
                          momPos.y + (momSize.height / 2) - (meSize.height / 2));
    }

    /*
     **************************************************************************
     * UTILITY METHODS.
     **************************************************************************
     * Select a save file, or return null.
     */
    public File selectOutputFile (String defaultName) {
        JFileChooser indexChooser = new JFileChooser("output");
        indexChooser.setFileFilter (new OutputFilter());
        indexChooser.setFileSelectionMode (JFileChooser.FILES_ONLY);
        indexChooser.setMultiSelectionEnabled (false);
        indexChooser.setApproveButtonText ("Save");
        indexChooser.setSelectedFile (new File (indexChooser.getCurrentDirectory(), defaultName));

        if (indexChooser.showOpenDialog(this) != JFileChooser.APPROVE_OPTION) {
            return null;
        }

        File outputFile = indexChooser.getSelectedFile();

        return outputFile;
    }

    private class OutputFilter extends FileFilter {
        public boolean accept (File f) {
            return (f.getName().endsWith(".html") || f.isDirectory());
        }

        public String getDescription() {
            return "HTML Report Output";
        }
    }

    /*
     **************************************************************************
     * Real methods.
     **************************************************************************
     */
    protected void doReportNow () {
        int whichReport = scriptCombo.getSelectedIndex();
        currentReport = null;

        Cursor waitCursor = new Cursor (Cursor.WAIT_CURSOR);
        Cursor defaultCursor = new Cursor (Cursor.DEFAULT_CURSOR);

        outputEditorPane.setText ("<Generating...>");

        this.setCursor (waitCursor);
        outputEditorPane.setCursor (waitCursor);

        try {
            Class generateHtmlParams[] = {Class.forName ("standup.parse.Index"), Class.forName ("java.lang.Integer")};
            Object generateHtmlArgs[] = {index, new Integer (reportFlavours[whichReport])};
            Class reportClass = Class.forName (reportClasses[whichReport]);

            currentReport = (AbstractReport) reportClass.newInstance();

            Method generateHtml = reportClass.getDeclaredMethod ("generateHtml", generateHtmlParams);

            reportContent = ((StringBuffer) generateHtml.invoke (currentReport, generateHtmlArgs));
        }
        catch (Throwable ex) {
            if (ex instanceof InvocationTargetException) {
                outputEditorPane.setText (((InvocationTargetException) ex).getTargetException().toString());

            } else {
                outputEditorPane.setText (ex.toString());
            }

            this.setCursor (defaultCursor);
            outputEditorPane.setCursor (defaultCursor);

            return;
        }

        /*
         * Write the output to a file, if there was any.
         */
        if (reportContent == null) {
            outputEditorPane.setText ("<ERROR: Report Has Null Content>");

        } else {
            try {
                tempFile = File.createTempFile ("standup", null, null);
                tempFile.deleteOnExit();
                BufferedWriter output = new BufferedWriter (new FileWriter (tempFile));

                output.write (new String (reportContent));
                output.close ();

                saveAsButton.setEnabled (true);
                displayButton.setEnabled (true);

                outputEditorPane.setText ("<Select Display or Save As...>");
            }
            catch (Throwable ex) {
                outputEditorPane.setText (ex.toString());
            }
        }

        this.setCursor (defaultCursor);
        outputEditorPane.setCursor (defaultCursor);
    }

    /*
     **************************************************************************
     * LISTENERS BELOW HERE
     **************************************************************************
     */
    /*
     **************************************************************************
     * Act on our button actions.
     */
    public void actionPerformed(ActionEvent e) {
        Object source = e.getSource();

        if (source == doneButton) {
            setVisible (false);

        /*
         * Start report?  OK, I got a bit fancy here, with the array of class
         * names.  It's a bit indulgent, but I do enjoy the reflection in Java,
         * and it does all come together very nicely.
         */
        } else if (source == generateButton) {
            doReportNow ();

        /*
         * Display report.
         */
        } else if (source == displayButton) {
            if (tempFile == null) {
                return;
            }
            try {
                outputEditorPane.setPage (tempFile.toURL());
            }
            catch (Throwable ex) {
                outputEditorPane.setText (ex.toString());
            }

        /*
         * Print output.  That's a good idea.
         */
        } else if (source == saveAsButton) {
            if (currentReport == null) {
                return;
            }
            File outputFile = selectOutputFile(currentReport.getDefaultName());
            if (outputFile == null) {
                return;
            }

            try {
                BufferedWriter output = new BufferedWriter (new FileWriter (outputFile));

                output.write (new String (reportContent));

                output.close ();
            }
            catch (IOException ioe) {
                // Nothing
            }
        }
    }
}