/******************************************************************************
 * Module Name : $RCSfile: AlphaDocument.java,v $
 *
 * Description : Utility class.
 *
 * Updates:
 *   $Log: AlphaDocument.java,v $
 *   Revision 1.4  2002/07/07 00:53:26  jcouper
 *   job:153
 *   Filenames must be lowercase, and special issues uppercase.
 *
 *   Revision 1.3  2001/10/11 21:43:51  jcouper
 *   134: Constrain filter text to one line and valid chars.  More to come.
 *
 *   Revision 1.2  1999/10/24 22:37:55  Jonathan
 *   Proper CVS headers.  Fix A-018/19/20
 *
 *
 * Copyright Notice:
 * (c) 1999. This source code was developed by Jonathan Couper and friends.
 *            You may use it for any purpose.  No guarantees given or implied.
 *****************************************************************************/
package standup.util;

import java.lang.Math;
import java.awt.Toolkit;
import javax.swing.text.JTextComponent;
import javax.swing.text.PlainDocument;
import javax.swing.text.Document;
import javax.swing.text.AttributeSet;
import javax.swing.text.BadLocationException;
import java.awt.event.FocusAdapter;
import java.awt.event.FocusEvent;

/**
*   Document class which can be plugged into any JTextComponent in
*   order to give it the ability to limit its text to accept digits only.
*   May also be optionally length limited (number of characters) and/or bounds
*   limited (numerical range).
*
*   @author Peter Ashford
*/
public class AlphaDocument extends PlainDocument {
    int maxLength;
    boolean lengthChecking  = false;
    boolean forceUpper = false;
    boolean forceLower = false;
    String extraAllowed = "";

    /**
    *   Sets the maximum number of characters to accept in this field and.
    *   turns length checking on.
    *
    *   @param  maxLen  maximum number of characters to limit the field to.
    */
    public void setMaxLength(int maxLen){
        maxLength       = maxLen;
        lengthChecking = true;
    }

    public void setExtraAllowed (String value) {
        extraAllowed = value;
    }

    public void setForceUpper (boolean value) {
        forceUpper = value;
        if (forceUpper) {
            forceLower = false;
        }
    }

    public void setForceLower (boolean value) {
        forceLower = value;
        if (forceLower) {
            forceUpper = false;
        }
    }

    /**
    *   Here to allow textFocusL to access the super class's InsertString
    *   method.
    */
    protected void superInsertString(String str){
        try{
            super.insertString(0, str, null);
        }
        catch(Exception e){
        }
    }

    /**
    *   We simply take input strings and only insert the alphanumeric content
    *   into the document
    */
    public void insertString(int offs, String str, AttributeSet a)
        throws BadLocationException {

        if (str == null || str.length()==0) {
            return;
        }

        if (forceUpper) {
            str = str.toUpperCase ();

        } else if (forceLower) {
            str = str.toLowerCase ();
        }

        char[] alphas = str.toCharArray();
        int j=0;

        // strip bad characters from digits[] array
        for (int i=0; i < alphas.length; i++) {
            if(Character.isLetterOrDigit(alphas[i]) ||
                (alphas[i] == '_') ||
                (alphas[i] == '-') ||
                (extraAllowed.indexOf (alphas[i]) >= 0)){

                alphas[j++] = alphas[i];
            }
            else{
                Toolkit.getDefaultToolkit().beep();
            }
        }


        if(j==0){
            return;
        }

        // perform length checking.
        if(lengthChecking){
            int l = str.length() + getLength();
            int diff = l - maxLength;

            if (diff>0){
                Toolkit.getDefaultToolkit().beep();
                return;
            }
        }

        super.insertString(offs, new String(alphas, 0, alphas.length), a);
    } // public void insertString(

} // class NumberDocument