/******************************************************************************
 * Module Name : $RCSfile: JAlphaTextField.java,v $
 *
 * Description : Utility class.
 *
 * Updates:
 *   $Log: JAlphaTextField.java,v $
 *   Revision 1.4  2002/07/07 00:53:26  jcouper
 *   job:153
 *   Filenames must be lowercase, and special issues uppercase.
 *
 *   Revision 1.3  2001/10/11 21:43:51  jcouper
 *   134: Constrain filter text to one line and valid chars.  More to come.
 *
 *   Revision 1.2  1999/10/24 22:37:56  Jonathan
 *   Proper CVS headers.  Fix A-018/19/20
 *
 *
 * Copyright Notice:
 * (c) 1999. This source code was developed by Jonathan Couper and friends.
 *            You may use it for any purpose.  No guarantees given or implied.
 *****************************************************************************/
package standup.util;

import javax.swing.text.Document;
import javax.swing.JTextField;

/**
*   Swing-based implementation of a alphanumeric-only text field. It supports most
*   of the standard JTextField constructors.  This text field issues a beep
*   with every non-valid character it receives.
*
*   @author Peter Ashford
*   @see    NumberDocument
*/
public class JAlphaTextField extends JTextField {
    AlphaDocument doc;

    /**
    *   Constructs a new JNumberTextField.  The field is empty and the number
    *   of columns is set to zero.
    */
    public JAlphaTextField() {
        super();
    }

    /**
    *   Constructs a new JAlphaTextField with the specified number of columns.
    *   The initial string is set to zero.
    *
    *   @param  columns     the desired number of columns for the text field.
    */
    public JAlphaTextField(int columns) {
        super(columns);
    }

    /**
    *   Constructs a new JAlphaTextField initialized with the specified text.  The
    *   column width is set to zero.
    *
    *   @param  text        the text to initialise the field with or null.
    */
    public JAlphaTextField(String text) {
        super(text);
    }

    /**
    *   Constructs a new JAlphaTextField initialized with the specified text and
    *   with the specified number of columns.
    *
    *   @param  text        the text to initialise the field with or null.
    *   @param  columns     the desired number of columns for the text field.
    */
    public JAlphaTextField(String text, int columns){
        super(text, columns);
    }

    /**
    *   Constructs a new JAlphaTextField initialized with the specified number
    *   of columns and length limited.
    *
    *   @param  columns     the desired number of columns for the text field.
    *   @param  length      the maximum number of characters to allow
    */
    public JAlphaTextField(int columns, int length){
        super(columns);
        setMaxLength(length);
    }


    /**
    *   Set the maximum character length of text to store in this field and.
    *   turn length checking on.
    *
    *   @param  length  maximum character length of field
    */
    public void setMaxLength(int length){
        doc.setMaxLength(length);
    }

    public void setExtraAllowed (String value) {
        doc.setExtraAllowed (value);
    }

    public void setForceUpper (boolean value) {
        doc.setForceUpper (value);
    }

    public void setForceLower (boolean value) {
        doc.setForceLower (value);
    }

    //--------------------------------------
    protected Document createDefaultModel() {
        if(doc==null){
            doc = new AlphaDocument();
        }
        return doc;
    }
}