/******************************************************************************
 * Module Name : $RCSfile: JLimitedTextField.java,v $
 *
 * Description : Utility class.
 *
 * Updates:
 *   $Log: JLimitedTextField.java,v $
 *   Revision 1.2  1999/10/24 22:37:56  Jonathan
 *   Proper CVS headers.  Fix A-018/19/20
 *
 *
 * Copyright Notice:
 * (c) 1999. This source code was developed by Jonathan Couper and friends. 
 * 	      	  You may use it for any purpose.  No guarantees given or implied.
 *****************************************************************************/ 
package standup.util;

import javax.swing.text.Document;
import javax.swing.event.DocumentListener;
import javax.swing.event.DocumentEvent;
import javax.swing.JTextField;
	
/**
*	Swing-based implementation of a length limited text field.  Issues a beep
*	and disallows further input when its specified limit is reached.
*
*	@author Peter Ashford
*  	@see    LimiteLengthDocument
*/
public class JLimitedTextField extends JTextField {
	
	private	int maxLength;				// the length text is limited to
	private int docLength;				// current length of text
	private LimitedLengthDocument doc;	// the document which ensures 
										// length-limited input
	
	/**
	* 	Constructs a new JLimitedTextField.  The field is empty, the number
	*	of diplayed columns is set to zero and the editiable text is limited 
	*	to the specified length.
	*
	*	@param	length	the maximum length of editable text.
	*/
	public JLimitedTextField(int length) {
		super();
		maxLength = length;
		doc.init(this, maxLength);
	}

	/**
	* 	Constructs a new JLimitedField with the specified number of displayed 
	*   columns and specified maximum text length.
	*	The initial string is set to zero.
	*
	*	@param	length		the maximum length of editable text.
	*	@param	columns		the desired number of columns for the text field.
	*/
	public JLimitedTextField(int length, int columns) {
		super(columns);	
		maxLength = length ;
		doc.init(this, maxLength);
	}

	//----------------------------------------------
	protected void fireActionPerformed(){
		super.fireActionPerformed();	
	}
	
	//----------------------------------------------
	protected Document createDefaultModel() {	
		if(doc==null){
			doc = new LimitedLengthDocument();
			doc.addDocumentListener(new limitedDocListener());
		}
		return doc;
	}
	
	//=============================================================
	class limitedDocListener implements DocumentListener{
		public void insertUpdate(DocumentEvent e){}
		public void removeUpdate(DocumentEvent e){}
		
		public void changedUpdate(DocumentEvent e){
			fireActionPerformed();
		}	
	
	} // limitDocListener

}

	