/******************************************************************************
 * Module Name : $RCSfile: JNumberTextField.java,v $
 *
 * Description : Utility class.
 *
 * Updates:
 *   $Log: JNumberTextField.java,v $
 *   Revision 1.4  2000/01/09 00:11:55  Jonathan
 *   Edit Dialog, Preferences
 *
 *   Revision 1.3  1999/10/24 22:37:56  Jonathan
 *   Proper CVS headers.  Fix A-018/19/20
 *
 *
 * Copyright Notice:
 * (c) 1999. This source code was developed by Jonathan Couper and friends. 
 * 	      	  You may use it for any purpose.  No guarantees given or implied.
 *****************************************************************************/ 
package standup.util;

import javax.swing.text.Document;
import javax.swing.JTextField;

/**
*	Swing-based implementation of a numbers-only text field. It supports most 
*	of the standard JTextField constructors.  This text field issues a beep 
*	with every non-decimal character it receives. 
*	Can optionally force input to be range-limited within a range specified
*	by two positive long integers.  Can be optionally character length limited.
*
*	@author Peter Ashford
*  	@see    NumberDocument
*/
public class JNumberTextField extends JTextField {
	NumberDocument doc;
	
	/**
	* 	Constructs a new JNumberTextField.  The field is empty and the number
	*	of columns is set to zero.
	*/
	public JNumberTextField() {
		super();
	}

	/**
	* 	Constructs a new JNumberField with the specified number of columns.
	*	The initial string is set to zero.
	*
	*	@param	columns		the desired number of columns for the text field.
	*/
	public JNumberTextField(int columns) {
		super(columns);
	}
	
	/**
	* 	Constructs a new JNumberTextField initialized with the specified text.  The
	*	column width is set to zero.
	*
	*	@param	text		the text to initialise the field with or null.
	*/
	public JNumberTextField(String text) {
		super(text);
	}
	
	/**
	* 	Constructs a new JNumberTextField initialized with the specified text and
	*	with the specified number of columns.
	*
	*	@param	text		the text to initialise the field with or null.
	*	@param	columns		the desired number of columns for the text field.
	*/
	public JNumberTextField(String text, int columns){
		super(text, columns);
	}
	
	/**
	* 	Constructs a new empty JNumberTextField with the specified bounds 
	*	limitation.
	*	If the user inserts text which would set the value to greater than
	*  	the maximum allowed value, to value will be set to the maximum 
	*	allowed (and vice versa with the minimum).
	*
	*	@param	minimum		the minimum decimal value to allow
	*	@param  maximum		the maximum decimal value to allow (>minimum)	
	*/
	public JNumberTextField(long minimum, long maximum){
		super();
		setBounds(minimum, maximum);
	}
	
	/**
	* 	Constructs a new JNumberTextField initialized with the specified number
	*	of columns and bounds-limited.
	*	If the user inserts text which would set the value to greater than
	*  	the maximum allowed value, to value will be set to the maximum 
	*	allowed (and vice versa with the minimum). Note that this behaviour
	*	may not work in field is length limited.
	*
	*	@param	columns		the desired number of columns for the text field.
	*	@param	minimum		the minimum decimal value to allow
	*	@param  maximum		the maximum decimal value to allow (>minimum)	
	*/
	public JNumberTextField(int columns, long minimum, long maximum){
		super(columns);
		setBounds(minimum, maximum);
	}
	
	
	/**
	* 	Constructs a new JNumberTextField initialized with the specified number
	*	of columns and length limited.
	*
	*	@param	columns		the desired number of columns for the text field.
	*	@param	length		the maximum number of characters to allow
	*/
	public JNumberTextField(int columns, int length){
		super(columns);
		setMaxLength(length);
	}
	
	
	/**
	*	Set bounds for integer bounds checking and turn bounds checking on.
	*	If the user inserts text which would set the value to greater than
	*  	the maximum allowed value, to value will be set to the maximum 
	*	allowed (and vice versa with the minimum).
	*
	*
	*	@param	minimum		the minimum decimal value to allow
	*	@param  maximum		the maximum decimal value to allow (>minimum)
	*/
	public void setBounds(long minimum, long maximum){
		doc.setMinMax(minimum, maximum, this);
	}

	/**
	*	Sets the "00 vs "0" flag.
	*
	*	@param	value   true or false.
	*/
	public void setCents(boolean value){
		doc.setCents (value);
	}

	
	/**
	*	Set the maximum character length of text to store in this field and.
	*	turn length checking on. 
	*
	*	@param	length	maximum character length of field
	*/
	public void setMaxLength(int length){
		doc.setMaxLength(length);
	}
	
	public Integer getInteger () {
		try {
			Integer integer = new Integer (getText());
			return integer;
		}
		catch (NumberFormatException nfe) {
			return null;
		}
	}
	
	public int getInt () {
		Integer integer = getInteger();
		if (integer == null) {
			return 0;

		} else {
			return integer.intValue();
		}
	}
	
	public boolean isNumeric () {
		return (getInteger() != null);
	}
		
		
	//--------------------------------------
	protected Document createDefaultModel() {
		if(doc==null){
			doc = new NumberDocument();
		}
		return doc;
	}	
}