/******************************************************************************
 * Module Name : $RCSfile: LeftToRightLayout.java,v $
 *
 * Description : Utility class.
 *
 * Updates:
 *   $Log: LeftToRightLayout.java,v $
 *   Revision 1.2  1999/10/24 22:37:56  Jonathan
 *   Proper CVS headers.  Fix A-018/19/20
 *
 *
 * Copyright Notice:
 * (c) 1999. This source code was developed by Jonathan Couper and friends. 
 * 	      	  You may use it for any purpose.  No guarantees given or implied.
 *****************************************************************************/ 
package standup.util;

import java.awt.*;
import java.awt.event.*;

public class LeftToRightLayout
implements LayoutManager2
{
    int hGap = 0;
    int vGap = 0;
    int alignment = ALIGN_STRETCH;
    int numRows = 1;
    int slackRow = 1;

	public LeftToRightLayout()
	{
	}

	public LeftToRightLayout (int alignmentIn)
	{
        super();
        alignment = alignmentIn;
	}

    public static final int ALIGN_TOP = 0;
    public static final int ALIGN_BOTTOM = 1;
    public static final int ALIGN_CENTER = 2;
    public static final int ALIGN_STRETCH = 3;
    public void setAlignment (int alignmentIn)
    {
        alignment = alignmentIn;
    }

    public void setHgap (int value) {
        hGap = value;
    }

    public void setVgap (int value) {
        vGap = value;
    }

    public void setRows (int numRowsIn, int slackRowIn) {
        if (numRowsIn > 0) {
            numRows = numRowsIn;

            if ((slackRowIn >= 1) && (slackRowIn <= numRows)) {
                slackRow = slackRowIn;

            } else {
                slackRow = numRows;
            }
        }
    }

	public void addLayoutComponent(String name, Component comp)
	{
	}

	public void removeLayoutComponent(Component comp)
	{
	}

	public Dimension preferredLayoutSize(Container parent)
	{
		int numComps = parent.getComponentCount();
		int totalWidth = 0;
        int rowHeight[] = new int[numRows];
/*
    Clear the row heights.
*/
        for (int a = 0; a < numRows; a++) {
            rowHeight[a] = 0;
        }
/*
    Loop and layout the components
*/
        int row = 0;
        int columnWidth = 0;
        Insets margins = parent.getInsets();
		for (int a = 0; (a < numComps); a++) {
    		Component c;
	    	Dimension d;

			c = parent.getComponent(a);
            if (c.isVisible()) {
                d = c.getPreferredSize();

                if ((row == 0) && (a > 0)) {
                    totalWidth = totalWidth + hGap;
                }
                if (d.height > rowHeight[row]) {
                    rowHeight[row] = d.height;
                }
                if (d.width > columnWidth) {
                    columnWidth = d.width;
                }
                row++;
                if ((row == numRows) || (a == (numComps - 1))) {
                    totalWidth = totalWidth + columnWidth;
                    columnWidth = 0;
                    row = 0;
                }
            }
		}
        totalWidth = totalWidth + margins.left + margins.right;
/*
    Add up the row heights.
*/
        int totalHeight = 0;
        for (int a = 0; a < numRows; a++) {
            if (a > 0) totalHeight = totalHeight + vGap;
            totalHeight = totalHeight + rowHeight[a];
        }
        totalHeight = totalHeight + margins.top + margins.bottom;

		Dimension retVal = new Dimension (totalWidth, totalHeight);

        return retVal;
	}

	public Dimension minimumLayoutSize(Container parent)
	{
    	return preferredLayoutSize (parent);
	}

	public void layoutContainer(Container parent)
	{
		int numComps = parent.getComponentCount();
		int totalWidth = 0;
        int rowHeight[] = new int[numRows];
/*
    Clear the rows.
*/
        for (int a = 0; a < numRows; a++) {
            rowHeight[a] = 0;
        }
/*
    Loop and measure the row heights.
*/
        int row = 0;
        int columnWidth = 0;
        Insets margins = parent.getInsets();
		for (int a = 0; (a < numComps); a++) {
    		Component c;
	    	Dimension d;

			c = parent.getComponent(a);
            if (c.isVisible()) {
                d = c.getPreferredSize();

                if ((row == 0) && (a > 0)) {
                    totalWidth = totalWidth + hGap;
                }
                if (d.height > rowHeight[row]) {
                    rowHeight[row] = d.height;
                }
                if (d.width > columnWidth) {
                    columnWidth = d.width;
                }

                row++;
                if ((row == numRows) || (a == (numComps - 1))) {
                    totalWidth = totalWidth + columnWidth;
                    columnWidth = 0;
                    row = 0;
                }
            }
		}
/*
    Stretch one of the rows if we don't fill the entire height.
*/
        int totalHeight = 0;
        for (int a = 0; a < numRows; a++) {
            if (a > 0) totalHeight = totalHeight + vGap;
            totalHeight = totalHeight + rowHeight[a];
        }
        totalHeight = totalHeight + margins.top + margins.bottom;

        int parentHeight = parent.getSize().height;
        if (totalHeight < parentHeight) {
            rowHeight[slackRow-1] = rowHeight[slackRow-1] + (parentHeight - totalHeight);
        }

/*
    Now loop again and size the components.
*/
        row = 0;
        columnWidth = 0;
        int x = margins.left;

		for (int a = 0; (a < numComps); a++) {
    		Component c;
	    	Dimension d;

			c = parent.getComponent(a);
            if (c.isVisible()) {
                d = c.getPreferredSize();
                if (alignment == ALIGN_STRETCH) { d.height = rowHeight[row]; }
                c.setSize (d);

                int y = margins.top;
                for (int i = 0; i < row; i++) {
                    y = y + vGap;
                    y = y + rowHeight[i];
                }
                if ((row == 0) && (a > 0)) {
                    x = x + hGap;
                }
                if (d.width > columnWidth) {
                    columnWidth = d.width;
                }
                if (alignment == ALIGN_BOTTOM) {
                    c.setLocation (x, y + (rowHeight[row] - d.height));
                } else if (alignment == ALIGN_CENTER) {
                    c.setLocation (x, y + (rowHeight[row] - d.height) / 2);
                } else {
                    c.setLocation (x, y);
                }

                row++;
                if ((row == numRows) || (a == (numComps - 1))) {
                    x = x + columnWidth;
                    columnWidth = 0;
                    row = 0;
                }
            }
		}
	}


	public void addLayoutComponent(Component c, Object constraints)
	{
	}

	public Dimension maximumLayoutSize(Container target)
	{
		return null;
	}
	public float getLayoutAlignmentX(Container target)
	{
		return 0;
	}
	public float getLayoutAlignmentY(Container target)
	{
		return 0;
	}
	public void invalidateLayout(Container target)
	{
	}
}