/******************************************************************************
 * Module Name : $RCSfile: ValueComponent.java,v $
 *
 * Description : Editor for CoverDate elements.
 *
 * Updates:
 *   $Log: ValueComponent.java,v $
 *   Revision 1.3  2002/07/14 03:48:33  jcouper
 *   job:159
 *   Option to over-ride the automatic value.
 *
 *   Revision 1.2  2002/07/07 02:45:50  jcouper
 *   job:156
 *   Cut & Paste for comic value field.
 *
 *   Revision 1.1  2000/01/10 02:24:55  Jonathan
 *   New utility components to simplify edit dialog.
 *
 *
 * Copyright Notice:
 * (c) 1999. This source code was developed by Jonathan Couper and friends.
 *            You may use it for any purpose.  No guarantees given or implied.
 *****************************************************************************/
package standup.util;

import java.awt.*;
import java.awt.event.*;
import java.awt.datatransfer.*;
import java.util.*;
import javax.swing.*;
import javax.swing.event.*;
import standup.*;
import standup.parse.*;
import standup.util.*;

public class ValueComponent
    extends JComponent
    implements DocumentListener, KeyListener, FocusListener, ClipboardOwner
{
    protected EventListenerList listenerList = new EventListenerList();

    double value = 0.0;

    TopToBottomLayout valueLayout = new TopToBottomLayout ();
    JLabel dollarLabel = new JLabel ("$");
    JNumberTextField dollarField = new JNumberTextField();
    JLabel dotLabel = new JLabel (".");
    JNumberTextField centsField = new JNumberTextField();

    public ValueComponent () {
        dollarField.setColumns (5);
        dollarField.setMaxLength (5);
        dollarField.setBounds (0, 99999);
        dollarField.setHorizontalAlignment (SwingConstants.RIGHT);
        centsField.setCents (true);
        centsField.setColumns (2);
        centsField.setMaxLength (2);
        centsField.setBounds (0, 99);

        valueLayout.setColumns (4, 2);

        setLayout (valueLayout);
        add (dollarLabel);
        add (dollarField);
        add (dotLabel);
        add (centsField);

        dollarField.getDocument().addDocumentListener (this);
        centsField.getDocument().addDocumentListener (this);
        dollarField.addKeyListener (this);
        centsField.addKeyListener (this);
        dollarField.addFocusListener (this);
        centsField.addFocusListener (this);
    }

    public ValueComponent (double newValue) {
        this ();

        setValue (newValue);

        configure ();
    }

    public void setEnabled (boolean state) {
        dollarField.setEnabled (state);
        centsField.setEnabled (state);
    }

    public void setValue (double newValue) {
        value = newValue;

        configure ();
    }

    public double getValue () {
        return value;
    }

    protected void configure () {
        long dollars = (long) value;
        long cents = Math.round (100 * (value - dollars));

        dollarField.setText (String.valueOf (dollars));

        if (cents >= 10) {
            centsField.setText (String.valueOf (cents));

        } else {
            centsField.setText ("0" + String.valueOf (cents));
        }
    }

    /*
     **************************************************************************
     * LISTENER FUNCTIONS
     **************************************************************************
     */
    /**
     * adds an ActionListener to the button
     */
    public void addActionListener(ActionListener l) {
        listenerList.add(ActionListener.class, l);
    }

    /**
     * removes an ActionListener from the button
     */
    public void removeActionListener(ActionListener l) {
        listenerList.remove(ActionListener.class, l);
    }

    /*
     * Notify all listeners that have registered interest for
     * notification on this event type.  The event instance
     * is lazily created using the parameters passed into
     * the fire method.
     * @see EventListenerList
     */
    protected void fireActionPerformed() {
        ActionEvent event = new ActionEvent (this, ActionEvent.ACTION_PERFORMED, "Change", 0);
        // Guaranteed to return a non-null array
        Object[] listeners = listenerList.getListenerList();
        // Process the listeners last to first, notifying
        // those that are interested in this event
        for (int i = listeners.length-2; i>=0; i-=2) {
            ((ActionListener)listeners[i+1]).actionPerformed(event);
        }
    }

    /*
     **************************************************************************
     * Document listeners.
     */
    public void changedUpdate (DocumentEvent e) {
        documentUpdate (e);
    }
    public void insertUpdate (DocumentEvent e) {
        documentUpdate (e);
    }
    public void removeUpdate (DocumentEvent e) {
        documentUpdate (e);
    }
    public void documentUpdate (DocumentEvent e) {
        double newValue = dollarField.getInt() + centsField.getInt() / 100.0;

        if (newValue != value) {
            value = newValue;
            fireActionPerformed ();
        }
    }

    /*
     **************************************************************************
     * Key listeners
     */
    public void keyPressed(KeyEvent e) {
        // Ctrl-C is copy.
        if ((e.getKeyCode() == KeyEvent.VK_C) && ((e.getModifiers() & KeyEvent.CTRL_MASK) != 0)) {
            getToolkit().getSystemClipboard().setContents (
                new StringSelection ("$" + dollarField.getText() + "." + centsField.getText()),
                this);

            // If we don't do this, the text field copy may supercede us
            e.consume();

        // Ctrl-V is paste.
        } else if ((e.getKeyCode() == KeyEvent.VK_V) && ((e.getModifiers() & KeyEvent.CTRL_MASK) != 0)) {
            Transferable transferable = getToolkit().getSystemClipboard().getContents (null);

            // Check we have some contents of the right kind
            if ((transferable == null) || !(transferable instanceof StringSelection)) {
                return;
            }

            // Turn our contents back into a string
            StringSelection valueStringSelection = (StringSelection) transferable;

            String valueAsText = "";
            try {
                valueAsText = (String) transferable.getTransferData (DataFlavor.stringFlavor);
            }
            catch (Exception ex) {
                return;
            }

            // Trim, strip any leading dollar
            valueAsText = valueAsText.trim();
            if (valueAsText.charAt (0) == '$') {
                valueAsText = valueAsText.substring (1);
            }

            // Split to dollars and cents
            try {
                if (valueAsText.indexOf(".") > -1) {
                    dollarField.setText (valueAsText.substring (0, valueAsText.indexOf(".")));
                    centsField.setText (valueAsText.substring (valueAsText.indexOf(".") + 1));

                } else {
                    dollarField.setText (valueAsText);
                    centsField.setText ("00");
                }
            }
            catch (Exception ex) {
                return;
            }

            // If we don't do this, the text field copy may supercede us
            e.consume();

            fireActionPerformed ();
        }
    }
    public void keyReleased(KeyEvent e) {}
    public void keyTyped(KeyEvent e) {}

    /*
     **************************************************************************
     * Focus listeners
     */
    public void focusGained(FocusEvent e) {
        if (e.getSource() == dollarField) {
            dollarField.selectAll();

        } else if (e.getSource() == centsField) {
            centsField.selectAll();
        }
    }
    public void focusLost(FocusEvent e) {}

    /*
     **************************************************************************
     * Clipboard listeners
     */
    public void lostOwnership (Clipboard clipboard, Transferable contents) {}
}
