/******************************************************************************
 * Module Name : $RCSfile: FullCardTableModel.java,v $
 *
 * Description : Table showing all the contents of a Card.
 *
 * Updates:
 *   $Log: FullCardTableModel.java,v $
 *   Revision 1.11  2003/01/23 06:37:46  jcouper
 *   Version 1.1 Changes
 *
 *   Revision 1.10  2002/07/13 02:22:40  jcouper
 *   job:157
 *   Still tweaking after changing grading and valuation.
 *
 *   Revision 1.9  2001/04/04 07:34:55  jcouper
 *   Support credits, filters, new columns, etc, etc.  Stable release.
 *
 *   Revision 1.8  2000/03/18 00:47:01  Jonathan
 *   Support control of which columns appear.
 *
 *   Revision 1.7  2000/01/10 02:22:54  Jonathan
 *   Use .00 for values.
 *
 *   Revision 1.6  2000/01/09 00:11:56  Jonathan
 *   Edit Dialog, Preferences
 *
 *   Revision 1.5  2000/01/07 05:43:34  Jonathan
 *   Render fixed cell dates differently
 *
 *   Revision 1.4  1999/10/24 22:37:56  Jonathan
 *   Proper CVS headers.  Fix A-018/19/20
 *
 *
 * Copyright Notice:
 * (c) 1999. This source code was developed by Jonathan Couper and friends.
 *            You may use it for any purpose.  No guarantees given or implied.
 *****************************************************************************/
package standup.views;

import java.util.*;
import javax.swing.*;
import javax.swing.table.*;
import standup.parse.*;
import standup.*;

public class FullCardTableModel
    extends AbstractTableModel
{
    protected static ApplicationSettings applicationSettings = null;

    /*
     * The column count is used internally.  The column percentage values
     * aren't used here, but this is just a convenient place to keep this
     * information in one place - we define the content, so let us define
     * the widths too.
     */
    protected static int COLUMN_COUNT = 7;
    protected static final int DATE_WIDTH = 100;
    protected static final int NAME_WIDTH = 200;
    protected static final int ISSUE_WIDTH = 70;
    protected static final int GRADE_WIDTH = 70;
    protected static final int NM_VALUE_WIDTH = 70;
    protected static final int VALUE_WIDTH = 70;
    protected static final int INFO_WIDTH = 280;
    protected static final int COMMENT_WIDTH = 280;
    protected static final int DATE_KEY = 0;
    protected static final int NAME_KEY = 1;
    protected static final int ISSUE_KEY = 2;
    protected static final int GRADE_KEY = 3;
    protected static final int NM_VALUE_KEY = 4;
    protected static final int VALUE_KEY = 5;
    protected static final int INFO_KEY = 6;
    protected static final int COMMENT_KEY = 7;

    protected static int prefWidths[] = {
        DATE_WIDTH,
        NAME_WIDTH,
        ISSUE_WIDTH,
        GRADE_WIDTH,
        NM_VALUE_WIDTH,
        VALUE_WIDTH,
        INFO_WIDTH,
        COMMENT_WIDTH};

    protected static int colKeys[] = {
        DATE_KEY,
        NAME_KEY,
        ISSUE_KEY,
        GRADE_KEY,
        NM_VALUE_KEY,
        VALUE_KEY,
        INFO_KEY,
        COMMENT_KEY};

    Card card = null;

    public FullCardTableModel (Card newCard) {
        if (applicationSettings == null) {
            applySettings ();
        }
        card = newCard;
    }

    public Card getCard () {
        return card;
    }

    public int getColumnCount() {
        return COLUMN_COUNT;
    }

    public int getRowCount() {
        return card.getTotalIssueCount ();
    }

    public static void applySettings () {
        applicationSettings = Application.getSettings ();

        COLUMN_COUNT = 0;
        if (applicationSettings.showDateColumn) {
            colKeys[COLUMN_COUNT] = DATE_KEY;
            prefWidths[COLUMN_COUNT++] = DATE_WIDTH;
        }

        colKeys[COLUMN_COUNT] = NAME_KEY;
        prefWidths[COLUMN_COUNT++] = NAME_WIDTH;

        colKeys[COLUMN_COUNT] = ISSUE_KEY;
        prefWidths[COLUMN_COUNT++] = ISSUE_WIDTH;

        if (applicationSettings.showGradeColumn) {
            colKeys[COLUMN_COUNT] = GRADE_KEY;
            prefWidths[COLUMN_COUNT++] = GRADE_WIDTH;
        }
        if (applicationSettings.showNmValueColumn) {
            colKeys[COLUMN_COUNT] = NM_VALUE_KEY;
            prefWidths[COLUMN_COUNT++] = NM_VALUE_WIDTH;
        }
        if (applicationSettings.showValueColumn) {
            colKeys[COLUMN_COUNT] = VALUE_KEY;
            prefWidths[COLUMN_COUNT++] = VALUE_WIDTH;
        }
        if (applicationSettings.showInfoColumn) {
            colKeys[COLUMN_COUNT] = INFO_KEY;
            prefWidths[COLUMN_COUNT++] = INFO_WIDTH;
        }
        if (applicationSettings.showCommentColumn) {
            colKeys[COLUMN_COUNT] = COMMENT_KEY;
            prefWidths[COLUMN_COUNT++] = COMMENT_WIDTH;
        }
    }

    /*
     * Return a string for our row/column.  This is our role in life.
     */
    public Object getValueAt(int row, int col) {
        Issue issue = card.issueAtArrayPos(row);
        TitleRef titleRef = card.titleRefAtArrayPos(row);

        switch (colKeys [col]) {
            case DATE_KEY: return issue.getCoverDate();
            case NAME_KEY: return titleRef.getAnyName();
            case ISSUE_KEY: return issue.getCode();
            case GRADE_KEY: return issue.getGradeAbbreviationOrBlank();
            case NM_VALUE_KEY:
                double nmValue = issue.getNmValue();
                long dollars = (long) nmValue;
                long cents = Math.round (100 * (nmValue - dollars));

                if ((dollars == 0) && (cents == 0)) {
                    return " ";

                } else {
                    return "$" + dollars + ((cents < 10) ? ".0" : ".") + cents;
                }

            case VALUE_KEY:
                String valueStr;

                double value = issue.getGradeValue();
                dollars = (long) value;
                cents = Math.round (100 * (value - dollars));

                if ((dollars == 0) && (cents == 0)) {
                    valueStr = " ";

                } else {
                    valueStr = "$" + dollars + ((cents < 10) ? ".0" : ".") + cents;
                }
                if (issue.isGradeValueOverride()) {
                    valueStr = valueStr + "*";
                }
                return valueStr;

            case INFO_KEY: return issue.getInfo();
            case COMMENT_KEY: return issue.getComment();

            default: return "Error";
        }
    }

    public String getColumnName (int column) {
        switch (colKeys [column]) {
            case DATE_KEY: return "Date";
            case NAME_KEY: return "Title Name";
            case ISSUE_KEY: return "Issue";
            case GRADE_KEY: return "Grade";
            case NM_VALUE_KEY: return "NM Val";
            case VALUE_KEY: return "Value";
            case INFO_KEY: return "Info";
            case COMMENT_KEY: return "Comment";

            default: return "Error";
        }
    }

    /*
     * Set the widths on the specified table to suit the data we provide.
     */
    public void setPreferredWidths (JTable table) {
        TableColumnModel columnModel = table.getColumnModel();

        /*
         * Set the columns to our preferred widths.
         */
        for (int i=0; i<COLUMN_COUNT; i++) {
            columnModel.getColumn(i).setPreferredWidth (prefWidths[i]);
        }
    }
}